@@ -51,7 +51,7 @@ class PlainBoxTool(LazyLoadingToolMixIn, PlainBoxToolBase):
                      self._load_config)),
             ('session', (p + "cmd_session:SessionCommand",
                          self._load_providers)),
-            ('device', p + "cmd_device:DeviceCommand",),
+            ('device', (p + "cmd_device:DeviceCommand",)),
             ('self-test', (p + "cmd_selftest:PlainboxSelfTestCommand",)),
             ('check-config', (p + "cmd_check_config:CheckConfigCommand",
                               self._load_config,)),
@@ -115,7 +115,8 @@ class StubBoxTool(PlainBoxTool):
     def _load_providers(self):
         logger.info("Loading stubbox provider...")
         from plainbox.impl.providers.special import get_stubbox
-        return [get_stubbox()]
+        from plainbox.impl.providers.special import get_exporters
+        return [get_stubbox(), get_exporters()]
 
 
 def main(argv=None):
