@@ -199,8 +199,12 @@ class NormalUI(IJobRunnerUI):
             'stdout': sys.stdout,
             'stderr': sys.stderr
         }[stream_name]
-        print(self.C.BLACK(line.decode("UTF-8", "ignore")),
-              end='', file=stream)
+        if stream_name == 'stdout':
+            print(self.C.GREEN(line.decode("UTF-8", "ignore")),
+                  end='', file=stream)
+        elif stream_name == 'stderr':
+            print(self.C.RED(line.decode("UTF-8", "ignore")),
+                  end='', file=stream)
         stream.flush()
 
     def finished_executing_program(self, returncode):
@@ -216,20 +220,26 @@ class NormalUI(IJobRunnerUI):
 
     def notify_about_purpose(self, job):
         if job.tr_purpose() is not None:
-            print(self.C.CYAN(_("Purpose:")))
+            print(self.C.WHITE(_("Purpose:")))
+            print()
             print(self.C.CYAN(job.tr_purpose()))
+            print()
         else:
             self.notify_about_description(job)
 
     def notify_about_steps(self, job):
         if job.tr_steps() is not None:
-            print(self.C.CYAN(_("Steps:")))
+            print(self.C.WHITE(_("Steps:")))
+            print()
             print(self.C.CYAN(job.tr_steps()))
+            print()
 
     def notify_about_verification(self, job):
         if job.tr_verification() is not None:
-            print(self.C.CYAN(_("Verification:")))
+            print(self.C.WHITE(_("Verification:")))
+            print()
             print(self.C.CYAN(job.tr_verification()))
+            print()
 
     def job_cannot_start(self, job, job_state, result):
         print(_("Job cannot be started because:"))
@@ -919,9 +929,17 @@ class RunInvocation(CheckBoxInvocationMixIn):
                 print("  " + _("comments") + ": {0}".format(
                     self.C.CYAN(result.comments, bright=False)))
             cmd = self._pick_action_cmd(allowed_actions)
+            # let's store new_comment early for verification if comment has
+            # already been added in the current UI step
+            new_comment = ''
             if cmd == 'set-pass':
                 result_builder.outcome = IJobResult.OUTCOME_PASS
             elif cmd == 'set-fail':
+                if 'explicit-fail' in job.get_flag_set() and not new_comment:
+                    new_comment = input(self.C.BLUE(
+                        _('Please enter your comments:') + '\n'))
+                    if new_comment:
+                        result_builder.add_comment(new_comment)
                 result_builder.outcome = IJobResult.OUTCOME_FAIL
             elif cmd == 'set-skip' or cmd is None:
                 result_builder.outcome = IJobResult.OUTCOME_SKIP
