@@ -100,12 +100,15 @@ class SpecialInvocation(CheckBoxInvocationMixIn):
                 print('\t"{}" [color=orange];'.format(job.id))
             for dep_id in job.get_direct_dependencies():
                 print('\t"{}" -> "{}";'.format(job.id, dep_id))
+            for dep_id in job.get_after_dependencies():
+                print('\t"{}" -> "{}" [style=dotted];'.format(job.id, dep_id))
             prog = job.get_resource_program()
             if ns.dot_resources and prog is not None:
                 for expression in prog.expression_list:
-                    print('\t"{}" [shape=ellipse,color=blue];'.format(
-                        expression.resource_id))
-                    print('\t"{}" -> "{}" [style=dashed, label="{}"];'.format(
-                        job.id, expression.resource_id,
-                        expression.text.replace('"', "'")))
+                    for resource_id in expression.resource_id_list:
+                        print('\t"{}" [shape=ellipse,color=blue];'.format(
+                            resource_id))
+                        print('\t"{}" -> "{}" [style=dashed, label="{}"];'.format(
+                            job.id, resource_id,
+                            expression.text.replace('"', "'")))
         print("}")
