@@ -1,13 +1,12 @@
 # This file is part of Checkbox.
 #
-# Copyright 2013 Canonical Ltd.
+# Copyright 2012-2014 Canonical Ltd.
 # Written by:
 #   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
 #
 # Checkbox is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3,
 # as published by the Free Software Foundation.
-
 #
 # Checkbox is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,19 +15,17 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
-
 """
 plainbox.impl.commands.test_parse
 =================================
 
 Test definitions for plainbox.impl.commands.parse module
 """
-
 import argparse
 from inspect import cleandoc
 from unittest import TestCase
 
-from plainbox.impl.commands.parse import ParseCommand
+from plainbox.impl.commands.cmd_parse import ParseCommand
 from plainbox.impl.parsers import ParserPlugIn
 from plainbox.impl.parsers import all_parsers
 from plainbox.testing_utils.io import TestIO
@@ -44,7 +41,7 @@ class TestParseCommand(TestCase):
         ParseCommand()
 
     _help = """
-usage: test parse [-h] PARSER-NAME
+usage: plainbox dev parse [-h] PARSER-NAME
 
 This command can be used to invoke any of the parsers exposed to the
 `plainbox.parsers` entry point, parse standard input and produce a JSON dump
@@ -85,7 +82,7 @@ Example: LANG=C pactl list | plainbox dev parse pactl-list
         # Ensure that a detailed help message was printed
         self.assertEqual(io.stdout, cleandoc(self._help) + '\n')
 
-    @mock.patch("plainbox.impl.commands.parse.ParseInvocation")
+    @mock.patch("plainbox.impl.commands.inv_parse.ParseInvocation")
     def test_invoked(self, patched_ParseInvocation):
         # Make a fake ParserPlugIn
         mock_parser = mock.Mock(spec=ParserPlugIn)
