@@ -37,7 +37,6 @@ from plainbox.impl.box import main
 from plainbox.impl.exporter.json import JSONSessionStateExporter
 from plainbox.impl.exporter.rfc822 import RFC822SessionStateExporter
 from plainbox.impl.exporter.text import TextSessionStateExporter
-from plainbox.impl.exporter.xml import XMLSessionStateExporter
 from plainbox.testing_utils.io import TestIO
 from plainbox.vendor.mock import patch, Mock
 
@@ -54,12 +53,6 @@ class TestRun(TestCase):
         self._sandbox = tempfile.mkdtemp()
         self._env = os.environ
         os.environ['XDG_CACHE_HOME'] = self._sandbox
-        self._exporters = OrderedDict([
-            ('json', JSONSessionStateExporter),
-            ('rfc822', RFC822SessionStateExporter),
-            ('text', TextSessionStateExporter),
-            ('xml', XMLSessionStateExporter),
-        ])
 
     def test_help(self):
         with TestIO(combined=True) as io:
@@ -135,30 +128,34 @@ class TestRun(TestCase):
     def test_output_format_list(self):
         with TestIO(combined=True) as io:
             with self.assertRaises(SystemExit) as call:
-                with patch('plainbox.impl.commands.inv_run.get_all_exporters') as mock_get_all_exporters:
-                    mock_get_all_exporters.return_value = self._exporters
-                    main(['run', '--output-format=?'])
+                main(['run', '--output-format=?'])
             self.assertEqual(call.exception.args, (0,))
         expected = """
-        Available output formats: json, rfc822, text, xml
+        Available output formats:
+        2013.com.canonical.plainbox::hexr - Generate XML (for certification)
+        2013.com.canonical.plainbox::html - Generate a standalone HTML
+        2013.com.canonical.plainbox::json - Generate JSON output
+        2013.com.canonical.plainbox::rfc822 - Generate RCF822 output
+        2013.com.canonical.plainbox::text - Generate plain text output
+        2013.com.canonical.plainbox::xlsx - Generate an Excel 2007+ XLSX document
         """
-        self.assertEqual(io.combined, cleandoc(expected) + "\n")
+        self.assertIn(cleandoc(expected) + "\n", io.combined)
 
     def test_output_option_list(self):
         with TestIO(combined=True) as io:
             with self.assertRaises(SystemExit) as call:
-                with patch('plainbox.impl.commands.inv_run.get_all_exporters') as mock_get_all_exporters:
-                    mock_get_all_exporters.return_value = self._exporters
-                    main(['run', '--output-option=?'])
+                main(['run', '--output-option=?'])
             self.assertEqual(call.exception.args, (0,))
         expected = """
         Each format may support a different set of options
-        json: with-io-log, squash-io-log, flatten-io-log, with-run-list, with-job-list, with-resource-map, with-job-defs, with-attachments, with-comments, with-job-via, with-job-hash, with-category-map, machine-json
-        rfc822: with-io-log, squash-io-log, flatten-io-log, with-run-list, with-job-list, with-resource-map, with-job-defs, with-attachments, with-comments, with-job-via, with-job-hash, with-category-map
-        text: with-io-log, squash-io-log, flatten-io-log, with-run-list, with-job-list, with-resource-map, with-job-defs, with-attachments, with-comments, with-job-via, with-job-hash, with-category-map
-        xml: client-name
+        2013.com.canonical.plainbox::hexr: 
+        2013.com.canonical.plainbox::html: 
+        2013.com.canonical.plainbox::json: with-io-log, squash-io-log, flatten-io-log, with-run-list, with-job-list, with-resource-map, with-job-defs, with-attachments, with-comments, with-job-via, with-job-hash, with-category-map, with-certification-status, machine-json
+        2013.com.canonical.plainbox::rfc822: with-io-log, squash-io-log, flatten-io-log, with-run-list, with-job-list, with-resource-map, with-job-defs, with-attachments, with-comments, with-job-via, with-job-hash, with-category-map, with-certification-status
+        2013.com.canonical.plainbox::text: with-io-log, squash-io-log, flatten-io-log, with-run-list, with-job-list, with-resource-map, with-job-defs, with-attachments, with-comments, with-job-via, with-job-hash, with-category-map, with-certification-status
+        2013.com.canonical.plainbox::xlsx: with-sys-info, with-summary, with-job-description, with-text-attachments, with-unit-categories
         """
-        self.assertEqual(io.combined, cleandoc(expected) + "\n")
+        self.assertIn(cleandoc(expected) + "\n", io.combined)
 
     def tearDown(self):
         shutil.rmtree(self._sandbox)
