@@ -122,19 +122,22 @@ class CheckBoxSessionStateController(ISessionStateController):
 
         Returns a set of pairs (dep_type, job_id) that describe all
         dependencies of the specified job. The first element in the pair,
-        dep_type, is either DEP_TYPE_DIRECT or DEP_TYPE_RESOURCE. The second
-        element is the id of the job.
+        dep_type, is either DEP_TYPE_DIRECT, DEP_TYPE_ORDERING or
+        DEP_TYPE_RESOURCE. The second element is the id of the job.
         """
         direct = DependencyMissingError.DEP_TYPE_DIRECT
+        ordering = DependencyMissingError.DEP_TYPE_ORDERING
         resource = DependencyMissingError.DEP_TYPE_RESOURCE
         direct_deps = job.get_direct_dependencies()
+        after_deps = job.get_after_dependencies()
         try:
             resource_deps = job.get_resource_dependencies()
         except ResourceProgramError:
             resource_deps = ()
         result = set(itertools.chain(
             zip(itertools.repeat(direct), direct_deps),
-            zip(itertools.repeat(resource), resource_deps)))
+            zip(itertools.repeat(resource), resource_deps),
+            zip(itertools.repeat(ordering), after_deps)))
         return result
 
     def get_inhibitor_list(self, session_state, job):
@@ -151,43 +154,48 @@ class CheckBoxSessionStateController(ISessionStateController):
         :returns:
             List of JobReadinessInhibitor
         """
+        inhibitors = []
         # Check if all job resource requirements are met
         prog = job.get_resource_program()
-        inhibitors = []
         if prog is not None:
             try:
                 prog.evaluate_or_raise(session_state.resource_map)
             except ExpressionCannotEvaluateError as exc:
-                # Lookup the related job (the job that provides the
-                # resources needed by the expression that cannot be
-                # evaluated)
-                related_job = session_state.job_state_map[
-                    exc.expression.resource_id].job
-                # Add A PENDING_RESOURCE inhibitor as we are unable to
-                # determine if the resource requirement is met or not. This
-                # can happen if the resource job did not ran for any reason
-                # (it can either be prevented from running by normal means
-                # or simply be on the run_list but just was not executed
-                # yet).
-                inhibitor = JobReadinessInhibitor(
-                    cause=InhibitionCause.PENDING_RESOURCE,
-                    related_job=related_job,
-                    related_expression=exc.expression)
-                inhibitors.append(inhibitor)
+                for resource_id in exc.expression.resource_id_list:
+                    if session_state.job_state_map[resource_id].result.outcome == 'pass':
+                        continue
+                    # Lookup the related job (the job that provides the
+                    # resources needed by the expression that cannot be
+                    # evaluated)
+                    related_job = session_state.job_state_map[resource_id].job
+                    # Add A PENDING_RESOURCE inhibitor as we are unable to
+                    # determine if the resource requirement is met or not. This
+                    # can happen if the resource job did not ran for any reason
+                    # (it can either be prevented from running by normal means
+                    # or simply be on the run_list but just was not executed
+                    # yet).
+                    inhibitor = JobReadinessInhibitor(
+                        cause=InhibitionCause.PENDING_RESOURCE,
+                        related_job=related_job,
+                        related_expression=exc.expression)
+                    inhibitors.append(inhibitor)
             except ExpressionFailedError as exc:
-                # Lookup the related job (the job that provides the
-                # resources needed by the expression that failed)
-                related_job = session_state.job_state_map[
-                    exc.expression.resource_id].job
-                # Add a FAILED_RESOURCE inhibitor as we have all the data
-                # to run the requirement program but it simply returns a
-                # non-True value. This typically indicates a missing
-                # software package or necessary hardware.
-                inhibitor = JobReadinessInhibitor(
-                    cause=InhibitionCause.FAILED_RESOURCE,
-                    related_job=related_job,
-                    related_expression=exc.expression)
-                inhibitors.append(inhibitor)
+                # When expressions fail then all the associated resources are
+                # marked as failed since we don't want to get into the analysis
+                # of logic expressions to know any "better".
+                for resource_id in exc.expression.resource_id_list:
+                    # Lookup the related job (the job that provides the
+                    # resources needed by the expression that failed)
+                    related_job = session_state.job_state_map[resource_id].job
+                    # Add a FAILED_RESOURCE inhibitor as we have all the data
+                    # to run the requirement program but it simply returns a
+                    # non-True value. This typically indicates a missing
+                    # software package or necessary hardware.
+                    inhibitor = JobReadinessInhibitor(
+                        cause=InhibitionCause.FAILED_RESOURCE,
+                        related_job=related_job,
+                        related_expression=exc.expression)
+                    inhibitors.append(inhibitor)
         # Check if all job dependencies ran successfully
         for dep_id in sorted(job.get_direct_dependencies()):
             dep_job_state = session_state.job_state_map[dep_id]
@@ -208,6 +216,16 @@ class CheckBoxSessionStateController(ISessionStateController):
                     cause=InhibitionCause.FAILED_DEP,
                     related_job=dep_job_state.job)
                 inhibitors.append(inhibitor)
+        # Check if all "after" dependencies ran yet
+        for dep_id in sorted(job.get_after_dependencies()):
+            dep_job_state = session_state.job_state_map[dep_id]
+            # If the dependency did not have a chance to run yet add the
+            # PENDING_DEP inhibitor.
+            if dep_job_state.result.outcome == IJobResult.OUTCOME_NONE:
+                inhibitor = JobReadinessInhibitor(
+                    cause=InhibitionCause.PENDING_DEP,
+                    related_job=dep_job_state.job)
+                inhibitors.append(inhibitor)
         return inhibitors
 
     def observe_result(self, session_state, job, result):
@@ -367,8 +385,8 @@ class CheckBoxSessionStateController(ISessionStateController):
                 via_cycle = get_via_cycle(job_state_map, added_job)
                 if via_cycle:
                     logger.warning(_("Automatically breaking via-cycle: %s"),
-                                    ' -> '.join(str(cycle_job)
-                                                for cycle_job in via_cycle))
+                                   ' -> '.join(str(cycle_job)
+                                               for cycle_job in via_cycle))
                     job_state_map[added_job.id].via_job = None
 
 
@@ -891,6 +909,8 @@ class QmlJobExecutionController(CheckBoxExecutionController):
 
     QML_SHELL_PATH = os.path.join(get_plainbox_dir(), 'data', 'qml-shell',
                                   'plainbox_qml_shell.qml')
+    QML_MODULES_PATH = os.path.join(get_plainbox_dir(), 'data',
+                                    'plainbox-qml-modules')
 
     def get_execution_command(self, job, job_state, config, session_dir,
                               nest_dir, shell_out_fd, shell_in_fd):
@@ -922,8 +942,9 @@ class QmlJobExecutionController(CheckBoxExecutionController):
             List of command arguments
 
         """
-        cmd = ['qmlscene', '--job', job.qml_file, '--fd-out', shell_out_fd,
-               '--fd-in', shell_in_fd, self.QML_SHELL_PATH]
+        cmd = ['qmlscene', '-I', self.QML_MODULES_PATH, '--job', job.qml_file,
+               '--fd-out', shell_out_fd, '--fd-in', shell_in_fd,
+               self.QML_SHELL_PATH]
         return cmd
 
     def get_checkbox_score(self, job):
@@ -1089,12 +1110,19 @@ class CheckBoxDifferentialExecutionController(CheckBoxExecutionController):
         base_env = os.environ
         target_env = super().get_execution_environment(
             job, job_state, config, session_dir, nest_dir)
-        return {
+        delta_env = {
             key: value
             for key, value in target_env.items()
             if key not in base_env or base_env[key] != value
             or key in job.get_environ_settings()
         }
+        # Neutral locale in the differential environment unless the
+        # 'preserve-locale' flag is set.
+        if 'preserve-locale' not in job.get_flag_set():
+            delta_env['LANG'] = 'C.UTF-8'
+            delta_env['LANGUAGE'] = ''
+            delta_env['LC_ALL'] = 'C.UTF-8'
+        return delta_env
 
     def get_execution_environment(self, job, job_state, config, session_dir,
                                   nest_dir):
