@@ -282,6 +282,14 @@ class LoggingHelper:
                 "formatter": "log_precise",
                 "delay": True,
             },
+            "logfile_bug": {
+                "class": "logging.handlers.RotatingFileHandler",
+                "filename": os.path.join(self.log_dir, "bug.log"),
+                "backupCount": 3,
+                "mode": "a",
+                "formatter": "log_precise",
+                "delay": True,
+            },
         }
 
     @property
@@ -352,6 +360,9 @@ class LoggingHelper:
                 "level": "ERROR",
                 "handlers": ["logfile_crash"],
             },
+            "plainbox.bug": {
+                "handlers": ["logfile_bug"],
+            },
         }
 
     @property
