@@ -207,7 +207,7 @@ class JobRunnerUIDelegate(extcmd.DelegateBase):
         Initialize the JobRunnerUIDelegate.
 
         :param ui:
-            (optional) an instnace of IJobRunnerUI to delegate events to
+            (optional) an instance of IJobRunnerUI to delegate events to
         """
         self.ui = ui
 
@@ -560,7 +560,8 @@ class JobRunner(IJobRunner):
         if job.plugin != "manual":
             # TRANSLATORS: please keep 'plugin' untranslated
             raise ValueError(_("bad job plugin value"))
-        return JobResultBuilder(outcome=IJobResult.OUTCOME_UNDECIDED).get_result()
+        return JobResultBuilder(
+            outcome=IJobResult.OUTCOME_UNDECIDED).get_result()
 
     def run_user_interact_job(self, job, job_state, config):
         """
@@ -733,9 +734,7 @@ class JobRunner(IJobRunner):
         delegate_cls = self._get_delegate_cls(config)
         extcmd_popen = delegate_cls(delegate)
         # Stream all IOLogRecord entries to disk
-        record_path = os.path.join(
-            self._jobs_io_log_dir, "{}.record.gz".format(
-                slugify(job.id)))
+        record_path = self.get_record_path_for_job(job)
         with gzip.open(record_path, mode='wb') as gzip_stream, \
                 io.TextIOWrapper(
                     gzip_stream, encoding='UTF-8') as record_stream:
@@ -770,6 +769,10 @@ class JobRunner(IJobRunner):
             execution_duration=execution_duration
         ).get_result()
 
+    def get_record_path_for_job(self, job):
+        return os.path.join(self._jobs_io_log_dir,
+                            "{}.record.gz".format(slugify(job.id)))
+
     def _get_dry_run_result(self, job):
         """
         Internal method of JobRunner.
@@ -965,7 +968,8 @@ class JobRunner(IJobRunner):
             Anyone listening to this signal does not need to remove any of the
             files. They are removed automatically after this method returns.
         """
-        if self._log_leftovers:
+        if (self._log_leftovers and
+                'has-leftovers' not in job.get_flag_set()):
             logger.warning(
                 _("Job {0} created leftover filesystem artefacts"
                   " in its working directory").format(job.id))
