@@ -123,7 +123,7 @@ class PlugIn(IPlugIn):
     and some arbitrary external object.
     """
 
-    def __init__(self, name: str, obj: object, load_time: float=0):
+    def __init__(self, name: str, obj: object, load_time: float=0, wrap_time: float=0):
         """
         Initialize the plug-in with the specified name and external object
 
@@ -133,10 +133,13 @@ class PlugIn(IPlugIn):
             The plugged in object itself
         :param load_time:
             Time it took to load the object (in fractional seconds)
+        :param wrap_time:
+            Time it took to wrap the object (in fractional seconds)
         """
         self._name = name
         self._obj = obj
         self._load_time = load_time
+        self._wrap_time = wrap_time
 
     def __repr__(self):
         return "<{!s} plugin_name:{!r}>".format(
@@ -167,10 +170,8 @@ class PlugIn(IPlugIn):
     def plugin_wrap_time(self) -> float:
         """
         time, in fractional seconds, that was needed to wrap the plugin
-
-        For this naive plugin class, this time is always zero.
         """
-        return 0
+        return self._wrap_time
 
 
 class IPlugInCollection(metaclass=abc.ABCMeta):
@@ -290,7 +291,6 @@ class PlugInCollectionBase(IPlugInCollection):
         self._wrapper_kwargs = wrapper_kwargs
         self._plugins = collections.OrderedDict()  # str -> IPlugIn instance
         self._loaded = False
-        self._mocked_objects = None
         self._problem_list = []
         self._discovery_time = 0
         if load:
@@ -369,8 +369,8 @@ class PlugInCollectionBase(IPlugInCollection):
         self._loaded = True
         self._plugins = collections.OrderedDict([
             (plugin.plugin_name, plugin)
-            for plugin in sorted(
-                plugins, key=lambda plugin: plugin.plugin_name)])
+            for plugin in plugins
+        ])
         if problem_list is None:
             problem_list = []
         self._problem_list = problem_list
@@ -564,14 +564,17 @@ class FsPlugInCollection(PlugInCollectionBase):
         for filename in filename_list:
             start_time = now()
             try:
-                with open(filename, encoding='UTF-8') as stream:
-                    text = stream.read()
+                text = self._get_file_text(filename)
             except (OSError, IOError) as exc:
                 logger.error(_("Unable to load %r: %s"), filename, str(exc))
                 self._problem_list.append(exc)
             else:
                 self.wrap_and_add_plugin(filename, text, now() - start_time)
 
+    def _get_file_text(self, filename):
+        with open(filename, encoding='UTF-8') as stream:
+            return stream.read()
+
     def _get_plugin_files(self):
         """
         Enumerate (generate) all plugin files according to 'path' and 'ext'
@@ -612,6 +615,62 @@ class FsPlugInCollection(PlugInCollectionBase):
                 yield info_file
 
 
+class LazyFileContent:
+    """
+    Support class for FsPlugInCollection's subclasses that behaves like a
+    string of text loaded from a file. The actual text is loaded on demand, the
+    first time it is needed.
+
+    The actual methods implemented here are just enough to work for loading a
+    provider. Since __getattr__() is implemented the class should be pretty
+    versatile but your millage may vary.
+    """
+
+    def __init__(self, name):
+        self.name = name
+        self._text = None
+
+    def __repr__(self):
+        return "<{} name:{!r}{}>".format(
+            self.__class__.__name__, self.name,
+            ' (pending)' if self._text is None else ' (loaded)')
+
+    def __str__(self):
+        self._ensure_loaded()
+        return self._text
+
+    def __iter__(self):
+        self._ensure_loaded()
+        return iter(self._text.splitlines(True))
+
+    def __getattr__(self, attr):
+        self._ensure_loaded()
+        return getattr(self._text, attr)
+
+    def _ensure_loaded(self):
+        if self._text is None:
+            with open(self.name, encoding='UTF-8') as stream:
+                self._text = stream.read()
+
+
+class LazyFsPlugInCollection(FsPlugInCollection):
+    """
+    Collection of plug-ins based on filesystem entries
+
+    Instantiate with :attr:`dir_list` and :attr:`ext`, call :meth:`load()` and
+    then access any of the loaded plug-ins using the API offered. All loaded
+    plugin information files are wrapped by a plug-in container. By default
+    that is :class:`PlugIn` but it may be adjusted if required.
+
+    The name of each plugin is the base name of the plugin file, the object of
+    each plugin is a handle that can be used to optionally load the content of
+    the file.
+    """
+
+    def _get_file_text(self, filename):
+        return LazyFileContent(filename)
+
+
 class LazyPlugInCollection(PlugInCollectionBase):
     """
     Collection of plug-ins based on a mapping of imported objects
@@ -627,9 +686,10 @@ class LazyPlugInCollection(PlugInCollectionBase):
         callbacks.
 
         :param callback_args_map:
-            any mapping from from any string (the plugin name) to a
-            tuple ("module:obj", *args) that if imported and called
-            ``obj(*args)`` produces the plugin object.
+            any mapping from from any string (the plugin name) to a tuple
+            ("module:obj", *args) that if imported and called ``obj(*args)``
+            produces the plugin object, alternatively, a mapping from the same
+            string to a string that is imported but *not* called.
         :param load:
             if true, load all of the plug-ins now
         :param wrapper:
@@ -680,7 +740,7 @@ class LazyPlugInCollection(PlugInCollectionBase):
             args = discovery_data[1:]
         else:
             callable_obj = discovery_data
-            args = ()
+            args = None
         if isinstance(callable_obj, str):
             logger.debug(_("Importing %s"),  callable_obj)
             callable_obj = getattr(
@@ -688,7 +748,10 @@ class LazyPlugInCollection(PlugInCollectionBase):
                     callable_obj.split(':', 1)[0], fromlist=[1]),
                 callable_obj.split(':', 1)[1])
         logger.debug(_("Calling %r with %r"), callable_obj, args)
-        return callable_obj(*args)
+        if args is None:
+            return callable_obj
+        else:
+            return callable_obj(*args)
 
     def get_all_names(self):
         """
@@ -697,7 +760,10 @@ class LazyPlugInCollection(PlugInCollectionBase):
         :returns:
             a list of plugin names
         """
-        return list(self._mapping.keys())
+        if self._loaded:
+            return super().get_all_names()
+        else:
+            return list(self._mapping.keys())
 
     def get_by_name(self, name):
         """
@@ -710,7 +776,9 @@ class LazyPlugInCollection(PlugInCollectionBase):
         :raises KeyError:
             if the specified name cannot be found
         """
-        if not self._loaded and name not in self._plugins:
+        if self._loaded:
+            return super().get_by_name(name)
+        if name not in self._plugins:
             discovery_data = self._mapping[name]
             self.load_one(name, discovery_data)
         return self._plugins[name]
@@ -730,3 +798,13 @@ class LazyPlugInCollection(PlugInCollectionBase):
             returns zero.
         """
         return self._discovery_time
+
+    @contextlib.contextmanager
+    def fake_plugins(self, plugins, problem_list=None):
+        old_mapping = self._mapping
+        self._mapping = {}  # fake the mapping
+        try:
+            with super().fake_plugins(plugins, problem_list):
+                yield
+        finally:
+            self._mapping = old_mapping
