@@ -98,7 +98,7 @@ class ExistingDirectoryValidatorTests(TestCase):
     def test_existing_directories_work(self, mock_isdir):
         mock_isdir.return_value = True
         self.assertEqual(self.validator(self.variable, self._PATH), None)
-        mock_isdir.assertCalledWith(self._PATH)
+        mock_isdir.assert_called_with(self._PATH)
 
     @mock.patch('os.path.isdir')
     def test_missing_directories_dont(self, mock_isdir):
@@ -106,7 +106,7 @@ class ExistingDirectoryValidatorTests(TestCase):
         self.assertEqual(
             self.validator(self.variable, self._PATH),
             "no such directory")
-        mock_isdir.assertCalledWith(self._PATH)
+        mock_isdir.assert_called_with(self._PATH)
 
 
 class AbsolutePathValidatorTests(TestCase):
@@ -667,7 +667,7 @@ class UnitPlugInTests(TestCase):
 
 class Provider1Tests(TestCase):
 
-    NAME = "2013.org.example:name"
+    NAME = "name"
     NAMESPACE = "2013.org.example"
     VERSION = "1.0"
     DESCRIPTION = "description"
@@ -685,7 +685,7 @@ class Provider1Tests(TestCase):
 
     def setUp(self):
         self.provider = Provider1(
-            self.NAME, self.VERSION, self.DESCRIPTION, self.SECURE,
+            self.NAME, self.NAMESPACE, self.VERSION, self.DESCRIPTION, self.SECURE,
             self.GETTEXT_DOMAIN, self.UNITS_DIR, self.JOBS_DIR,
             self.WHITELISTS_DIR, self.DATA_DIR, self.BIN_DIR, self.LOCALE_DIR,
             self.BASE_DIR,
@@ -701,7 +701,7 @@ class Provider1Tests(TestCase):
     def test_repr(self):
         self.assertEqual(
             repr(self.provider),
-            "<Provider1 name:'2013.org.example:name'>")
+            "<Provider1 name:'name'>")
 
     def test_name(self):
         """
@@ -809,12 +809,22 @@ class Provider1Tests(TestCase):
         fake_content = [
             PlugIn(self.JOBS_DIR + "/path/to/jobs1.txt", (
                 "id: a2\n"
+                "plugin: shell\n"
+                "command: true\n"
                 "\n"
-                "id: a1\n")),
+                "id: a1\n"
+                "plugin: shell\n"
+                "command: true\n"
+            )),
             PlugIn(self.JOBS_DIR + "/path/to/jobs2.txt", (
                 "id: a3\n"
+                "plugin: shell\n"
+                "command: true\n"
                 "\n"
-                "id: a4\n"))]
+                "id: a4\n"
+                "plugin: shell\n"
+                "command: true\n"
+            ))]
         fake_problems = [IOError("first problem"), OSError("second problem")]
         with self.provider.fake(fake_content, fake_problems):
             job_list = self.provider.job_list
@@ -878,7 +888,7 @@ class Provider1Tests(TestCase):
         circumstances
         """
         Provider1(
-            self.NAME, self.VERSION, self.DESCRIPTION, self.SECURE,
+            self.NAME, self.NAMESPACE, self.VERSION, self.DESCRIPTION, self.SECURE,
             self.GETTEXT_DOMAIN, self.UNITS_DIR, self.JOBS_DIR,
             self.WHITELISTS_DIR, self.DATA_DIR, self.BIN_DIR, self.LOCALE_DIR,
             self.BASE_DIR)
@@ -892,7 +902,7 @@ class Provider1Tests(TestCase):
         circumstances
         """
         Provider1(
-            self.NAME, self.VERSION, self.DESCRIPTION, self.SECURE,
+            self.NAME, self.NAMESPACE, self.VERSION, self.DESCRIPTION, self.SECURE,
             self.GETTEXT_DOMAIN, self.UNITS_DIR, self.JOBS_DIR,
             self.WHITELISTS_DIR, self.DATA_DIR, self.BIN_DIR, locale_dir=None,
             base_dir=self.BASE_DIR)
