@@ -27,6 +27,8 @@ import os
 from plainbox.abc import IProvider1
 from plainbox.i18n import gettext as _
 from plainbox.impl.secure.config import Config, Variable
+from plainbox.impl.secure.config import (
+    ValidationError as ConfigValidationError)
 from plainbox.impl.secure.config import IValidator
 from plainbox.impl.secure.config import NotEmptyValidator
 from plainbox.impl.secure.config import NotUnsetValidator
@@ -71,8 +73,8 @@ class ProviderContentPlugIn(PlugIn):
     """
 
     def __init__(self, filename, text, load_time, provider, *,
-                 validate=True, validation_kwargs=None,
-                 check=False, context=None):
+                 validate=False, validation_kwargs=None,
+                 check=True, context=None):
         start_time = now()
         try:
             # Inspect the file
@@ -205,8 +207,8 @@ class WhiteListPlugIn(ProviderContentPlugIn):
     # NOTE: This version of __init__() exists solely so that provider can
     # default to None. This is still used in some places and must be supported.
     def __init__(self, filename, text, load_time, provider=None, *,
-                 validate=True, validation_kwargs=None,
-                 check=False, context=None):
+                 validate=False, validation_kwargs=None,
+                 check=True, context=None):
         super().__init__(
             filename, text, load_time, provider, validate=validate,
             validation_kwargs=validation_kwargs, check=check, context=context)
@@ -307,9 +309,10 @@ class UnitPlugIn(ProviderContentPlugIn):
     ) -> "Iterable[WhiteList]":
         for unit in (unit for unit in inspect_result
                      if unit.Meta.name == 'test plan'):
-            yield WhiteList(
-                unit.include, name=unit.partial_id, origin=unit.origin,
-                implicit_namespace=unit.provider.namespace)
+            if unit.include is not None:
+                yield WhiteList(
+                    unit.include, name=unit.partial_id, origin=unit.origin,
+                    implicit_namespace=unit.provider.namespace)
 
     # NOTE: this version of plugin_object() is just for legacy code support
     @property
@@ -557,8 +560,8 @@ class ProviderContentClassifier:
 
     def _classify_whitelist(self, filename: str):
         """ classify .whitelist files in whitelist_dir as whitelist """
-        if (filename.startswith(self.provider.whitelists_dir)
-                and filename.endswith(".whitelist")):
+        if (filename.startswith(self.provider.whitelists_dir) and
+                filename.endswith(".whitelist")):
             return (FileRole.legacy_whitelist, self.provider.whitelists_dir,
                     WhiteListPlugIn)
 
@@ -570,8 +573,8 @@ class ProviderContentClassifier:
 
     def _classify_exec(self, filename: str):
         """ classify files in bin_dir as scripts/executables """
-        if (filename.startswith(self.provider.bin_dir)
-                and os.access(filename, os.F_OK | os.X_OK)):
+        if (filename.startswith(self.provider.bin_dir) and
+                os.access(filename, os.F_OK | os.X_OK)):
             with open(filename, 'rb') as stream:
                 chunk = stream.read(2)
             role = FileRole.script if chunk == b'#!' else FileRole.binary
@@ -579,9 +582,9 @@ class ProviderContentClassifier:
 
     def _classify_built_exec(self, filename: str):
         """ classify files in build_bin_dir as scripts/executables """
-        if (filename.startswith(self.provider.build_bin_dir)
-                and os.access(filename, os.F_OK | os.X_OK)
-                and os.path.basename(filename) in self.EXECUTABLES):
+        if (filename.startswith(self.provider.build_bin_dir) and
+                os.access(filename, os.F_OK | os.X_OK) and
+                os.path.basename(filename) in self.EXECUTABLES):
             with open(filename, 'rb') as stream:
                 chunk = stream.read(2)
             role = FileRole.script if chunk == b'#!' else FileRole.binary
@@ -589,8 +592,8 @@ class ProviderContentClassifier:
 
     def _classify_built_i18n(self, filename: str):
         """ classify files in build_mo_dir as i18n """
-        if (filename.startswith(self.provider.build_mo_dir)
-                and os.path.splitext(filename)[1] == '.mo'):
+        if (filename.startswith(self.provider.build_mo_dir) and
+                os.path.splitext(filename)[1] == '.mo'):
             return (FileRole.i18n, self.provider.build_bin_dir,
                     ProviderContentPlugIn)
 
@@ -617,9 +620,9 @@ class ProviderContentClassifier:
             return (FileRole.manage_py, self.provider.base_dir, None)
 
     def _classify_po(self, filename: str):
-        if (os.path.dirname(filename) == self.provider.po_dir
-            and (os.path.splitext(filename)[1] in ('.po', '.pot')
-                 or os.path.basename(filename) == 'POTFILES.in')):
+        if (os.path.dirname(filename) == self.provider.po_dir and
+                (os.path.splitext(filename)[1] in ('.po', '.pot') or
+                 os.path.basename(filename) == 'POTFILES.in')):
             return (FileRole.src, self.provider.base_dir, None)
 
     def _classify_src(self, filename: str):
@@ -727,16 +730,19 @@ class Provider1(IProvider1):
     number of fields involved in basic initialization.
     """
 
-    def __init__(self, name, version, description, secure, gettext_domain,
-                 units_dir, jobs_dir, whitelists_dir, data_dir, bin_dir,
-                 locale_dir, base_dir, *, validate=True,
-                 validation_kwargs=None, check=False, context=None):
+    def __init__(self, name, namespace, version, description, secure,
+                 gettext_domain, units_dir, jobs_dir, whitelists_dir, data_dir,
+                 bin_dir, locale_dir, base_dir, *, validate=False,
+                 validation_kwargs=None, check=True, context=None):
         """
         Initialize a provider with a set of meta-data and directories.
 
         :param name:
             provider name / ID
 
+        :param namespace:
+            provider namespace
+
         :param version:
             provider version
 
@@ -790,7 +796,13 @@ class Provider1(IProvider1):
             this is a single argument. This is a keyword-only argument.
         """
         # Meta-data
+        if namespace is None:
+            namespace = name.split(':', 1)[0]
+            self._has_dedicated_namespace = False
+        else:
+            self._has_dedicated_namespace = True
         self._name = name
+        self._namespace = namespace
         self._version = version
         self._description = description
         self._secure = secure
@@ -833,7 +845,7 @@ class Provider1(IProvider1):
 
     @classmethod
     def from_definition(cls, definition, secure, *,
-                        validate=True, validation_kwargs=None, check=False,
+                        validate=False, validation_kwargs=None, check=True,
                         context=None):
         """
         Initialize a provider from Provider1Definition object
@@ -864,8 +876,9 @@ class Provider1(IProvider1):
         logger.debug("Loading provider from definition %r", definition)
         # Initialize the provider object
         return cls(
-            definition.name, definition.version, definition.description,
-            secure, definition.effective_gettext_domain,
+            definition.name, definition.namespace or None, definition.version,
+            definition.description, secure,
+            definition.effective_gettext_domain,
             definition.effective_units_dir, definition.effective_jobs_dir,
             definition.effective_whitelists_dir, definition.effective_data_dir,
             definition.effective_bin_dir, definition.effective_locale_dir,
@@ -901,7 +914,12 @@ class Provider1(IProvider1):
         year-domain pair and allows to create private namespaces by using
         sub-domains.
         """
-        return self._name.split(':', 1)[0]
+        return self._namespace
+
+    @property
+    def has_dedicated_namespace(self):
+        """Flag set if namespace was defined by a dedicated field."""
+        return self._has_dedicated_namespace
 
     @property
     def version(self):
@@ -1104,8 +1122,8 @@ class Provider1(IProvider1):
         """
         return sorted(
             unit.path for unit in self.unit_list
-            if unit.Meta.name == 'file'
-            and unit.role in (FileRole.script, FileRole.binary))
+            if unit.Meta.name == 'file' and
+            unit.role in (FileRole.script, FileRole.binary))
 
     @property
     def whitelist_list(self):
@@ -1223,6 +1241,35 @@ class IQNValidator(PatternValidator):
             return _("must look like RFC3720 IQN")
 
 
+class ProviderNameValidator(PatternValidator):
+
+    """
+    Validator for the provider name.
+
+    Two forms are allowed:
+
+        - short form (requires a separate namespace definition)
+        - verbose form (based on RFC3720 IQN-like strings)
+
+    The short form is supposed to look like Debian package name.
+    """
+
+    _PATTERN = (
+        "^"
+        "([0-9]{4}\.[a-z][a-z0-9-]*(\.[a-z][a-z0-9-]*)+:[a-z][a-z0-9-]*)"
+        "|"
+        "([a-z0-9-]+)"
+        "$"
+    )
+
+    def __init__(self):
+        super().__init__(self._PATTERN)
+
+    def __call__(self, variable, new_value):
+        if super().__call__(variable, new_value):
+            return _("must look like RFC3720 IQN")
+
+
 class VersionValidator(PatternValidator):
     """
     A validator for provider provider version.
@@ -1274,6 +1321,14 @@ class Provider1Definition(Config):
         can be Unset and their validators only check values other than Unset.
     """
 
+    # NOTE: See the implementation note in :class:`Provider1PluginIn` to
+    # understand the effect of this flag.
+    relocatable = Variable(
+        section='PlainBox Provider',
+        help_text=_("Flag indicating if the provider is relocatable"),
+        kind=bool,
+    )
+
     location = Variable(
         section='PlainBox Provider',
         help_text=_("Base directory with provider data"),
@@ -1290,12 +1345,23 @@ class Provider1Definition(Config):
         validator_list=[
             NotUnsetValidator(),
             NotEmptyValidator(),
-            IQNValidator(),
+            ProviderNameValidator(),
+        ])
+
+    namespace = Variable(
+        section='PlainBox Provider',
+        help_text=_("Namespace of the provider"),
+        validator_list=[
+            # NOTE: it *can* be unset, then name must be IQN
+            NotEmptyValidator(),
         ])
 
     @property
     def name_without_colon(self):
-        return self.name.replace(':', '.')
+        if ':' in self.name:
+            return self.name.replace(':', '.')
+        else:
+            return self.name
 
     version = Variable(
         section='PlainBox Provider',
@@ -1560,6 +1626,23 @@ class Provider1Definition(Config):
         if implicit2 is not None and os.path.isdir(implicit2):
             return implicit2
 
+    def validate_whole(self):
+        """
+        Validate the provider definition object.
+
+        :raises ValidationError:
+            If the namespace is not defined and name is using a simplified
+            format that doesn't contain an embedded namespace part.
+        """
+        super().validate_whole()
+        if not self.namespace:
+            variable = self.__class__.name
+            value = self.name
+            validator = IQNValidator()
+            message = validator(variable, value)
+            if message is not None:
+                raise ConfigValidationError(variable, value, message)
+
 
 class Provider1PlugIn(PlugIn):
     """
@@ -1577,6 +1660,19 @@ class Provider1PlugIn(PlugIn):
         definition = Provider1Definition()
         # Load the provider definition
         definition.read_string(definition_text)
+        # If the relocatable flag is set, set location to the base directory of
+        # the filename and reset all the other directories (to Unset). This is
+        # to allow creation of .provider files that can be moved entirely, and
+        # as long as they follow the implicit source layout, they will work
+        # okay.
+        if definition.relocatable:
+            definition.location = os.path.dirname(filename)
+            definition.units_dir = Unset
+            definition.jobs_dir = Unset
+            definition.whitelists_dir = Unset
+            definition.data_dir = Unset
+            definition.bin_dir = Unset
+            definition.locale_dir = Unset
         # any validation issues prevent plugin from being used
         if definition.problem_list:
             # take the earliest problem and report it
