@@ -18,6 +18,8 @@
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
 
 """
+Session resume handling.
+
 :mod:`plainbox.impl.session.resume` -- session resume handling
 ==============================================================
 
@@ -66,14 +68,20 @@ logger = logging.getLogger("plainbox.session.resume")
 
 
 class SessionResumeError(Exception):
+
     """
+    Base for all session resume exceptions.
+
     Base class for exceptions that can be raised when attempting to
     resume a dormant session.
     """
 
 
 class CorruptedSessionError(SessionResumeError):
+
     """
+    Exception raised when suspended session is corrupted.
+
     Exception raised when :class:`SessionResumeHelper` cannot decode
     the session byte stream. This exception will be raised with additional
     context that captures the actual underlying cause. Having this exception
@@ -82,7 +90,10 @@ class CorruptedSessionError(SessionResumeError):
 
 
 class IncompatibleSessionError(SessionResumeError):
+
     """
+    Exception raised when suspended session is correct but incompatible.
+
     Exception raised when :class:`SessionResumeHelper` comes across malformed
     or unsupported data that was (presumably) produced by
     :class:`SessionSuspendHelper`
@@ -90,14 +101,20 @@ class IncompatibleSessionError(SessionResumeError):
 
 
 class IncompatibleJobError(SessionResumeError):
+
     """
+    Exception raised when suspended session needs a different version of a job.
+
     Exception raised when :class:`SessionResumeHelper` detects that the set of
     jobs it knows about is incompatible with what was saved before.
     """
 
 
 class BrokenReferenceToExternalFile(SessionResumeError):
+
     """
+    Exception raised when suspended session needs an external file that's gone.
+
     Exception raised when :class:`SessionResumeHelper` detects that a file
     needed by the session to resume is not present. This is typically used to
     signal inaccessible log files.
@@ -105,13 +122,18 @@ class BrokenReferenceToExternalFile(SessionResumeError):
 
 
 class EnvelopeUnpackMixIn:
+
     """
-    A mix-in class capable of unpacking the envelope of the session storage
+    A mix-in class capable of unpacking the envelope of the session storage.
+
+    This class assists in unpacking the "envelope" in which the session data is
+    actually stored. The envelope is simply gzip but other kinds of envelope
+    can be added later.
     """
 
     def unpack_envelope(self, data):
         """
-        Unpack the binary envelope and get access to a JSON object
+        Unpack the binary envelope and get access to a JSON object.
 
         :param data:
             Bytes representing the dormant session
@@ -136,6 +158,8 @@ class EnvelopeUnpackMixIn:
 
 class SessionPeekHelper(EnvelopeUnpackMixIn):
 
+    """A helper class to peek at session state meta-data quickly."""
+
     def peek(self, data):
         """
         Peek at the meta-data of a dormant session.
@@ -174,12 +198,15 @@ class SessionPeekHelper(EnvelopeUnpackMixIn):
             return SessionPeekHelper4().peek_json(json_repr)
         elif version == 5:
             return SessionPeekHelper5().peek_json(json_repr)
+        elif version == 6:
+            return SessionPeekHelper6().peek_json(json_repr)
         else:
             raise IncompatibleSessionError(
                 _("Unsupported version {}").format(version))
 
 
 class SessionResumeHelper(EnvelopeUnpackMixIn):
+
     """
     Helper class for implementing session resume feature.
 
@@ -298,6 +325,9 @@ class SessionResumeHelper(EnvelopeUnpackMixIn):
         elif version == 5:
             helper = SessionResumeHelper5(
                 self.job_list, self.flags, self.location)
+        elif version == 6:
+            helper = SessionResumeHelper6(
+                self.job_list, self.flags, self.location)
         else:
             raise IncompatibleSessionError(
                 _("Unsupported version {}").format(version))
@@ -305,24 +335,38 @@ class SessionResumeHelper(EnvelopeUnpackMixIn):
 
 
 class ResumeDiscardQualifier(SimpleQualifier):
+
     """
+    Qualifier for jobs that need to be discarded after resume.
+
     A job qualifier that designates jobs that should be removed
     after doing a session resume.
     """
 
     def __init__(self, retain_id_set):
+        """
+        Initialize the qualifier.
+
+        :param retain_id_set:
+            The set of job identifiers that should be retained on resume.
+        """
         super().__init__(Origin.get_caller_origin())
         self._retain_id_set = frozenset(retain_id_set)
 
     def get_simple_match(self, job):
+        """Check if a job should be listed by this qualifier."""
         return job.id not in self._retain_id_set
 
 
 class MetaDataHelper1MixIn:
 
+    """Mix-in class for working with v1 meta-data."""
+
     @classmethod
     def _restore_SessionState_metadata(cls, metadata, session_repr):
         """
+        Reconstruct the session state meta-data.
+
         Extract meta-data information from the representation of the session
         and set it in the given session object
         """
@@ -345,9 +389,13 @@ class MetaDataHelper1MixIn:
 
 class MetaDataHelper2MixIn(MetaDataHelper1MixIn):
 
+    """Mix-in class for working with v2 meta-data."""
+
     @classmethod
     def _restore_SessionState_metadata(cls, metadata, session_repr):
         """
+        Reconstruct the session state meta-data.
+
         Extract meta-data information from the representation of the session
         and set it in the given session object
         """
@@ -374,9 +422,13 @@ class MetaDataHelper2MixIn(MetaDataHelper1MixIn):
 
 class MetaDataHelper3MixIn(MetaDataHelper2MixIn):
 
+    """Mix-in class for working with v3 meta-data."""
+
     @classmethod
     def _restore_SessionState_metadata(cls, metadata, session_repr):
         """
+        Reconstruct the session state meta-data.
+
         Extract meta-data information from the representation of the session
         and set it in the given session object
         """
@@ -390,8 +442,9 @@ class MetaDataHelper3MixIn(MetaDataHelper2MixIn):
 
 
 class SessionPeekHelper1(MetaDataHelper1MixIn):
+
     """
-    Helper class for implementing session peek feature
+    Helper class for implementing session peek feature.
 
     This class works with data constructed by
     :class:`~plainbox.impl.session.suspend.SessionSuspendHelper1` which has
@@ -429,8 +482,9 @@ class SessionPeekHelper1(MetaDataHelper1MixIn):
 
 
 class SessionPeekHelper2(MetaDataHelper2MixIn, SessionPeekHelper1):
+
     """
-    Helper class for implementing session peek feature
+    Helper class for implementing session peek feature.
 
     This class works with data constructed by
     :class:`~plainbox.impl.session.suspend.SessionSuspendHelper1` which has
@@ -442,8 +496,9 @@ class SessionPeekHelper2(MetaDataHelper2MixIn, SessionPeekHelper1):
 
 
 class SessionPeekHelper3(MetaDataHelper3MixIn, SessionPeekHelper2):
+
     """
-    Helper class for implementing session peek feature
+    Helper class for implementing session peek feature.
 
     This class works with data constructed by
     :class:`~plainbox.impl.session.suspend.SessionSuspendHelper1` which has
@@ -455,8 +510,9 @@ class SessionPeekHelper3(MetaDataHelper3MixIn, SessionPeekHelper2):
 
 
 class SessionPeekHelper4(SessionPeekHelper3):
+
     """
-    Helper class for implementing session peek feature
+    Helper class for implementing session peek feature.
 
     This class works with data constructed by
     :class:`~plainbox.impl.session.suspend.SessionSuspendHelper1` which has
@@ -468,8 +524,9 @@ class SessionPeekHelper4(SessionPeekHelper3):
 
 
 class SessionPeekHelper5(SessionPeekHelper4):
+
     """
-    Helper class for implementing session peek feature
+    Helper class for implementing session peek feature.
 
     This class works with data constructed by
     :class:`~plainbox.impl.session.suspend.SessionSuspendHelper5` which has
@@ -480,9 +537,23 @@ class SessionPeekHelper5(SessionPeekHelper4):
     """
 
 
-class SessionResumeHelper1(MetaDataHelper1MixIn):
+class SessionPeekHelper6(SessionPeekHelper5):
     """
-    Helper class for implementing session resume feature
+    Helper class for implementing session peek feature
+
+    This class works with data constructed by
+    :class:`~plainbox.impl.session.suspend.SessionSuspendHelper6` which has
+    been pre-processed by :class:`SessionPeekHelper` (to strip the initial
+    envelope).
+
+    The only goal of this class is to reconstruct session state meta-data.
+    """
+
+
+class SessionResumeHelper1(MetaDataHelper1MixIn):
+
+    """
+    Helper class for implementing session resume feature.
 
     This class works with data constructed by
     :class:`~plainbox.impl.session.suspend.SessionSuspendHelper1` which has
@@ -660,7 +731,7 @@ class SessionResumeHelper1(MetaDataHelper1MixIn):
 
     def _process_job(self, session, jobs_repr, results_repr, job_id):
         """
-        Process all representation details associated with a particular job
+        Process all representation details associated with a particular job.
 
         This method takes a session object, representation of all the jobs
         and all the results (and a job id) and tries to reconstruct the
@@ -715,6 +786,8 @@ class SessionResumeHelper1(MetaDataHelper1MixIn):
     @classmethod
     def _restore_SessionState_desired_job_list(cls, session, session_repr):
         """
+        Reconstruct the list of desired jobs.
+
         Extract the representation of desired_job_list from the session and
         set it back to the session object. This method should be called after
         all the jobs are discovered.
@@ -741,9 +814,40 @@ class SessionResumeHelper1(MetaDataHelper1MixIn):
                 _("'desired_job_list' refers to unknown job {!r}").format(
                     exc.args[0]))
 
+    @classmethod
+    def _restore_SessionState_mandatory_job_list(cls, session, session_repr):
+        """
+        Extract the representation of mandatory_job_list from the session and
+        set it back to the session object. This method should be called after
+        all the jobs are discovered.
+
+        :raises CorruptedSessionError:
+            if mandatory_job_list refers to unknown job
+        """
+        # List of all the _ids_ of the jobs that were selected
+        mandatory_job_list = [
+            _validate(
+                job_id, value_type=str,
+                value_type_msg=_("Each job id must be a string"))
+            for job_id in _validate(
+                session_repr, key='mandatory_job_list', value_type=list)]
+        # Restore job selection
+        logger.debug(
+            _("calling update_mandatory_job_list(%r)"), mandatory_job_list)
+        try:
+            session.update_mandatory_job_list([
+                session.job_state_map[job_id].job
+                for job_id in mandatory_job_list])
+        except KeyError as exc:
+            raise CorruptedSessionError(
+                _("'mandatory_job_list' refers to unknown job {!r}").format(
+                    exc.args[0]))
+
     @classmethod
     def _restore_SessionState_job_list(cls, session, session_repr):
         """
+        Reconstruct the list of known jobs.
+
         Trim job_list so that it has only those jobs that are mentioned by the
         session representation. This should never fail as anything that might
         go wrong must have gone wrong before.
@@ -777,6 +881,8 @@ class SessionResumeHelper1(MetaDataHelper1MixIn):
     @classmethod
     def _build_JobResult(cls, result_repr, flags, location):
         """
+        Reconstruct a single job result.
+
         Convert the representation of MemoryJobResult or DiskJobResult
         back into an actual instance.
         """
@@ -841,9 +947,7 @@ class SessionResumeHelper1(MetaDataHelper1MixIn):
 
     @classmethod
     def _build_IOLogRecord(cls, record_repr):
-        """
-        Convert the representation of IOLogRecord back the the object
-        """
+        """Convert the representation of IOLogRecord back the object."""
         _validate(record_repr, value_type=list)
         delay = _validate(record_repr, key=0, value_type=float)
         if delay < 0:
@@ -870,8 +974,9 @@ class SessionResumeHelper1(MetaDataHelper1MixIn):
 
 
 class SessionResumeHelper2(MetaDataHelper2MixIn, SessionResumeHelper1):
+
     """
-    Helper class for implementing session resume feature
+    Helper class for implementing session resume feature.
 
     This class works with data constructed by
     :class:`~plainbox.impl.session.suspend.SessionSuspendHelper2` which has
@@ -889,8 +994,9 @@ class SessionResumeHelper2(MetaDataHelper2MixIn, SessionResumeHelper1):
 
 
 class SessionResumeHelper3(MetaDataHelper3MixIn, SessionResumeHelper2):
+
     """
-    Helper class for implementing session resume feature
+    Helper class for implementing session resume feature.
 
     This class works with data constructed by
     :class:`~plainbox.impl.session.suspend.SessionSuspendHelper3` which has
@@ -908,8 +1014,9 @@ class SessionResumeHelper3(MetaDataHelper3MixIn, SessionResumeHelper2):
 
 
 class SessionResumeHelper4(SessionResumeHelper3):
+
     """
-    Helper class for implementing session resume feature
+    Helper class for implementing session resume feature.
 
     This class works with data constructed by
     :class:`~plainbox.impl.session.suspend.SessionSuspendHelper4` which has
@@ -927,8 +1034,9 @@ class SessionResumeHelper4(SessionResumeHelper3):
 
 
 class SessionResumeHelper5(SessionResumeHelper4):
+
     """
-    Helper class for implementing session resume feature
+    Helper class for implementing session resume feature.
 
     This class works with data constructed by
     :class:`~plainbox.impl.session.suspend.SessionSuspendHelper5` which has
@@ -954,11 +1062,65 @@ class SessionResumeHelper5(SessionResumeHelper4):
             raise ValueError("Location must be a directory name")
         return os.path.join(location, io_log_filename)
 
+class SessionResumeHelper6(SessionResumeHelper5):
+    """
+    Helper class for implementing session resume feature
+
+    This class works with data constructed by
+    :class:`~plainbox.impl.session.suspend.SessionSuspendHelper5` which has
+    been pre-processed by :class:`SessionResumeHelper` (to strip the initial
+    envelope).
+
+    Due to the constraints of what can be represented in a suspended session,
+    this class cannot work in isolation. It must operate with a list of know
+    jobs.
+
+    Since (most of the) jobs are being provided externally (as they represent
+    the non-serialized parts of checkbox or other job providers) several
+    failure modes are possible. Those are documented in :meth:`resume()`
+    """
+
+    def _build_SessionState(self, session_repr, early_cb=None):
+        """
+        Reconstruct the session state object.
+
+        This method creates a fresh SessionState instance and restores
+        jobs, results, meta-data and desired job list using helper methods.
+        """
+        # Construct a fresh session object.
+        session = SessionState(self.job_list)
+        logger.debug(_("Constructed new session for resume %r"), session)
+        # Give early_cb a chance to see the session before we start resuming.
+        # This way applications can see, among other things, generated jobs
+        # as they are added to the session, by registering appropriate signal
+        # handlers on the freshly-constructed session instance.
+        if early_cb is not None:
+            logger.debug(_("Invoking early callback %r"), early_cb)
+            new_session = early_cb(session)
+            if new_session is not None:
+                logger.debug(
+                    _("Using different session for resume: %r"), new_session)
+                session = new_session
+        # Restore bits and pieces of state
+        logger.debug(
+            _("Starting to restore jobs and results to %r..."), session)
+        self._restore_SessionState_jobs_and_results(session, session_repr)
+        logger.debug(_("Starting to restore metadata..."))
+        self._restore_SessionState_metadata(session.metadata, session_repr)
+        logger.debug(_("restored metadata %r"), session.metadata)
+        logger.debug(_("Starting to restore mandatory job list..."))
+        self._restore_SessionState_mandatory_job_list(session, session_repr)
+        logger.debug(_("Starting to restore desired job list..."))
+        self._restore_SessionState_desired_job_list(session, session_repr)
+        logger.debug(_("Starting to restore job list..."))
+        self._restore_SessionState_job_list(session, session_repr)
+        # Return whatever we've got
+        logger.debug(_("Resume complete!"))
+        return session
+
 
 def _validate(obj, **flags):
-    """
-    Multi-purpose extraction and validation function.
-    """
+    """Multi-purpose extraction and validation function."""
     # Fetch data from the container OR use json_repr directly
     if 'key' in flags:
         key = flags['key']
