@@ -31,9 +31,11 @@ from plainbox.impl import deprecated
 from plainbox.impl.depmgr import DependencyDuplicateError
 from plainbox.impl.depmgr import DependencyError
 from plainbox.impl.depmgr import DependencySolver
+from plainbox.impl.secure.qualifiers import select_jobs
 from plainbox.impl.session.jobs import JobState
 from plainbox.impl.session.jobs import UndesiredJobReadinessInhibitor
 from plainbox.impl.unit.job import JobDefinition
+from plainbox.impl.unit.unit_with_id import UnitWithId
 from plainbox.impl.unit.testplan import TestPlanUnitSupport
 from plainbox.vendor import morris
 
@@ -69,7 +71,7 @@ class SessionMetaData:
 
     def __init__(self, title=None, flags=None, running_job_name=None,
                  app_blob=None, app_id=None):
-        """ Initialize a new session state meta-data object. """
+        """Initialize a new session state meta-data object."""
         if flags is None:
             flags = []
         self._title = title
@@ -79,7 +81,7 @@ class SessionMetaData:
         self._app_id = app_id
 
     def __repr__(self):
-        """ Get the representation of the session state meta-data. """
+        """Get the representation of the session state meta-data."""
         return "<{} title:{!r} flags:{!r} running_job_name:{!r}>".format(
             self.__class__.__name__, self.title, self.flags,
             self.running_job_name)
@@ -98,7 +100,7 @@ class SessionMetaData:
 
     @title.setter
     def title(self, title):
-        """ set the session title to the given value. """
+        """set the session title to the given value."""
         self._title = title
 
     @property
@@ -117,7 +119,7 @@ class SessionMetaData:
 
     @flags.setter
     def flags(self, flags):
-        """ set the session flags to the given set. """
+        """set the session flags to the given set."""
         self._flags = flags
 
     @property
@@ -141,7 +143,7 @@ class SessionMetaData:
 
     @running_job_name.setter
     def running_job_name(self, running_job_name):
-        """ set the id of the running job. """
+        """set the id of the running job."""
         self._running_job_name = running_job_name
 
     @property
@@ -159,7 +161,7 @@ class SessionMetaData:
 
     @app_blob.setter
     def app_blob(self, value):
-        """ set the application specific binary blob to the given value. """
+        """set the application specific binary blob to the given value."""
         if value is not None and not isinstance(value, bytes):
             # TRANSLATORS: please don't translate app_blob, None and bytes
             raise TypeError(_("app_blob must be either None or bytes"))
@@ -177,7 +179,7 @@ class SessionMetaData:
 
     @app_id.setter
     def app_id(self, value):
-        """ Set the application identifier to the given value. """
+        """Set the application identifier to the given value."""
         if value is not None and not isinstance(value, str):
             # TRANSLATORS: please don't translate app_blob, None and bytes
             raise TypeError(_("app_id must be either None or str"))
@@ -242,6 +244,7 @@ class SessionDeviceContext:
             self._unit_list = []
             self._provider_list = []
             self._state = SessionState(self._unit_list)
+            self._unit_id_map = {}
         else:
             if not isinstance(state, SessionState):
                 raise TypeError
@@ -252,6 +255,9 @@ class SessionDeviceContext:
                 unit.provider for unit in self._unit_list
             })
             self._state = state
+            self._unit_id_map = {unit.id: unit for unit in state.unit_list if
+                                 isinstance(unit, UnitWithId)}
+
         self._test_plan_list = []
         # Connect SessionState's signals to fire our signals. This
         # way all manipulation done through the SessionState object
@@ -361,6 +367,8 @@ class SessionDeviceContext:
         self._test_plan_list = test_plan_list
         self._invalidate_override_map()
         self._bulk_override_update()
+        if test_plan_list:
+            self._update_mandatory_job_list()
 
     def add_provider(self, provider, add_units=True):
         """
@@ -425,6 +433,26 @@ class SessionDeviceContext:
         # NOTE: no need to fire the on_unit_removed() signal because the state
         # object and we've connected it to will fire our version.
 
+    def get_unit(self, unit_id, kind_name=None):
+        """
+        Get an unit with a specific identifier.
+
+        :param unit_id:
+            The identifier of the unit to find
+        :param kind_name:
+            (optional) Name of the type of unit.  By default units of any type
+            can be found. Unit kind is the value of the ``unit.Meta.name``
+            attribute.  Using this argument allows the caller to quickly find
+            only units of a particular type without having to do the filtering
+            on their side.
+        :raises KeyError:
+            If the matching unit does not exists.
+        """
+        unit = self._unit_id_map[unit_id]
+        if kind_name is not None and unit.Meta.name != kind_name:
+            raise KeyError(unit_id)
+        return unit
+
     def get_ctrl_for_job(self, job):
         """
         Get the execution controller most applicable to run this job.
@@ -460,7 +488,7 @@ class SessionDeviceContext:
 
     @morris.signal
     def on_provider_added(self, provider):
-        """ Signal sent whenever a provider is added to the context. """
+        """Signal sent whenever a provider is added to the context."""
         logger.info(_("Provider %s added to context %s"), provider, self)
         # Invalidate the list of execution controllers as they depend
         # on the accuracy of provider_list
@@ -468,20 +496,24 @@ class SessionDeviceContext:
 
     @morris.signal
     def on_unit_added(self, unit):
-        """ Signal sent whenever a unit is added to the context. """
+        """Signal sent whenever a unit is added to the context."""
         logger.debug(_("Unit %s added to context %s"), unit, self)
         if unit.Meta.name == 'job':
             self.on_job_added(unit)
+        if isinstance(unit, UnitWithId):
+            self._unit_id_map[unit.id] = unit
 
     @morris.signal
     def on_job_added(self, job):
-        """ Signal sent whenever a new job unit is added to the context. """
+        """Signal sent whenever a new job unit is added to the context."""
         self._override_update(job)
 
     @morris.signal
     def on_unit_removed(self, unit):
-        """ Signal sent whenever a unit is removed from the context. """
+        """Signal sent whenever a unit is removed from the context."""
         logger.debug(_("Unit %s removed from context %s"), unit, self)
+        if isinstance(unit, UnitWithId):
+            del self._unit_id_map[unit.id]
 
     def compute_shared(self, cache_key, func, *args, **kwargs):
         """
@@ -508,12 +540,12 @@ class SessionDeviceContext:
         return self._shared_cache[cache_key]
 
     def invalidate_shared(self, cache_key):
-        """ Invalidate a cached shared value. """
+        """Invalidate a cached shared value."""
         if cache_key in self._shared_cache:
             del self._shared_cache[cache_key]
 
     def _compute_execution_ctrl_list(self):
-        """ Compute the list of execution controllers. """
+        """Compute the list of execution controllers."""
         # TODO: tie this with the upcoming device patches
         import sys
         if sys.platform == 'linux':
@@ -536,11 +568,11 @@ class SessionDeviceContext:
             return []
 
     def _invalidate_execution_ctrl_list(self, *args, **kwargs):
-        """ Invalidate the list of execution controllers. """
+        """Invalidate the list of execution controllers."""
         self.invalidate_shared(self._CACHE_EXECUTION_CTRL_LIST)
 
     def _compute_override_map(self):
-        """ Compute the map of field overrides. """
+        """Compute the map of field overrides."""
         override_map = collections.defaultdict(list)
         for test_plan in self._test_plan_list:
             support = TestPlanUnitSupport(test_plan)
@@ -549,7 +581,7 @@ class SessionDeviceContext:
         return override_map
 
     def _invalidate_override_map(self, *args, **kwargs):
-        """ Invalidate the cached field override map. """
+        """Invalidate the cached field override map."""
         self.invalidate_shared(self._CACHE_OVERRIDE_MAP)
 
     def _bulk_override_update(self):
@@ -569,6 +601,15 @@ class SessionDeviceContext:
             if re.match(pattern, job.id):
                 job_state.apply_overrides(override_list)
 
+    def _update_mandatory_job_list(self):
+        qualifier_list = []
+        for test_plan in self._test_plan_list:
+            qualifier_list.append(test_plan.get_mandatory_qualifier())
+        mandatory_job_list = select_jobs(
+            self.state.job_list, qualifier_list)
+        self.state.update_mandatory_job_list(mandatory_job_list)
+        self.state.update_desired_job_list(self.state.desired_job_list)
+
 
 class SessionState:
 
@@ -695,11 +736,11 @@ class SessionState:
 
     @morris.signal
     def on_unit_added(self, unit):
-        """ Signal sent whenever a unit is added to the session. """
+        """Signal sent whenever a unit is added to the session."""
 
     @morris.signal
     def on_unit_removed(self, unit):
-        """ Signal sent whenever a unit is removed from the session. """
+        """Signal sent whenever a unit is removed from the session."""
 
     def __init__(self, unit_list):
         """
@@ -746,6 +787,7 @@ class SessionState:
         self._job_state_map = {job.id: JobState(job)
                                for job in self._job_list}
         self._desired_job_list = []
+        self._mandatory_job_list = []
         self._run_list = []
         self._resource_map = {}
         self._metadata = SessionMetaData()
@@ -818,6 +860,17 @@ class SessionState:
                 self.on_job_removed(job)
                 self.on_unit_removed(job)
 
+    def update_mandatory_job_list(self, mandatory_job_list):
+        """
+        Update the set of mandatory jobs (that must run).
+
+        This method simply stores the list of mandatory jobs inside the session
+        state. The next time the set of desired jobs is altered via a call
+        :meth:`update_desired_job_list()` the effective selection will also
+        include mandatory jobs.
+        """
+        self._mandatory_job_list = mandatory_job_list
+
     def update_desired_job_list(self, desired_job_list):
         """
         Update the set of desired jobs (that ought to run).
@@ -834,7 +887,8 @@ class SessionState:
         """
         # Remember a copy of original desired job list. We may modify this list
         # so let's not mess up data passed by the caller.
-        self._desired_job_list = list(desired_job_list)
+        self._desired_job_list = list(
+            desired_job_list + self._mandatory_job_list)
         # Reset run list just in case desired_job_list is empty
         self._run_list = []
         # Try to solve the dependency graph. This is done in a loop as may need
@@ -850,7 +904,7 @@ class SessionState:
             # resources or runtime complexity.
             try:
                 self._run_list = DependencySolver.resolve_dependencies(
-                    job_list, self._desired_job_list)
+                    job_list, self.mandatory_job_list + self._desired_job_list)
             except DependencyError as exc:
                 # When a dependency error is detected remove the affected job
                 # form _desired_job_list and try again.
@@ -1098,9 +1152,19 @@ class SessionState:
         """
         return self._job_list
 
+    @property
+    def mandatory_job_list(self):
+        """
+        List of all mandatory jobs that must run.
+
+        Testplan units can specify a list of jobs that have to be run and are
+        not supposed to be deselected by the application user.
+        """
+        return self._mandatory_job_list
+
     @property
     def unit_list(self):
-        """ List of all known units. """
+        """List of all known units."""
         return self._unit_list
 
     @property
@@ -1127,12 +1191,12 @@ class SessionState:
 
     @property
     def job_state_map(self):
-        """ Map from job id to JobState associated with each job. """
+        """Map from job id to JobState associated with each job."""
         return self._job_state_map
 
     @property
     def resource_map(self):
-        """ Map from resource id to a list of resource records. """
+        """Map from resource id to a list of resource records."""
         return self._resource_map
 
     def get_outcome_stats(self):
@@ -1172,15 +1236,16 @@ class SessionState:
             certification status value.
         """
         return {
-            job_id: job_state for job_id, job_state in
-            self.job_state_map.items()
-            if job_state.result.outcome in outcome_filter
-            and job_state.effective_certification_status
-            in certification_status_filter}
+            job_id: job_state
+            for job_id, job_state in self.job_state_map.items()
+            if (job_state.result.outcome in outcome_filter and
+                job_state.effective_certification_status in
+                certification_status_filter)
+        }
 
     @property
     def metadata(self):
-        """ meta-data object associated with this session state. """
+        """meta-data object associated with this session state."""
         return self._metadata
 
     def _recompute_job_readiness(self):
