@@ -1,13 +1,12 @@
 # This file is part of Checkbox.
 #
-# Copyright 2012, 2013 Canonical Ltd.
+# Copyright 2012-2015 Canonical Ltd.
 # Written by:
 #   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
 #
 # Checkbox is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3,
 # as published by the Free Software Foundation.
-
 #
 # Checkbox is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,46 +15,53 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
-
 """
 plainbox.impl.test_session
 ==========================
 
 Test definitions for plainbox.impl.session module
 """
-
+from doctest import DocTestSuite
+from doctest import REPORT_NDIFF
 from unittest import TestCase, expectedFailure
 
 from plainbox.abc import IJobResult
+from plainbox.impl.session import InhibitionCause
 from plainbox.impl.session import JobReadinessInhibitor
 from plainbox.impl.session import JobState
 from plainbox.impl.session import UndesiredJobReadinessInhibitor
 from plainbox.impl.testing_utils import make_job, make_job_result
 
 
+def load_tests(loader, tests, ignore):
+    tests.addTests(DocTestSuite(
+        'plainbox.impl.session.jobs', optionflags=REPORT_NDIFF))
+    return tests
+
+
 class JobReadinessInhibitorTests(TestCase):
 
     def test_bad_initialization(self):
         self.assertRaises(ValueError, JobReadinessInhibitor,
-                          JobReadinessInhibitor.UNDESIRED - 1)
+                          InhibitionCause.UNDESIRED - 1)
         self.assertRaises(ValueError, JobReadinessInhibitor,
-                          JobReadinessInhibitor.FAILED_RESOURCE + 1)
+                          InhibitionCause.FAILED_RESOURCE + 1)
         self.assertRaises(ValueError, JobReadinessInhibitor,
-                          JobReadinessInhibitor.PENDING_DEP)
+                          InhibitionCause.PENDING_DEP)
         self.assertRaises(ValueError, JobReadinessInhibitor,
-                          JobReadinessInhibitor.FAILED_DEP)
+                          InhibitionCause.FAILED_DEP)
         self.assertRaises(ValueError, JobReadinessInhibitor,
-                          JobReadinessInhibitor.PENDING_RESOURCE)
+                          InhibitionCause.PENDING_RESOURCE)
         self.assertRaises(ValueError, JobReadinessInhibitor,
-                          JobReadinessInhibitor.FAILED_RESOURCE)
+                          InhibitionCause.FAILED_RESOURCE)
         job = make_job("A")
         self.assertRaises(ValueError, JobReadinessInhibitor,
-                          JobReadinessInhibitor.PENDING_RESOURCE, job)
+                          InhibitionCause.PENDING_RESOURCE, job)
         self.assertRaises(ValueError, JobReadinessInhibitor,
-                          JobReadinessInhibitor.FAILED_RESOURCE, job)
+                          InhibitionCause.FAILED_RESOURCE, job)
 
     def test_unknown(self):
-        obj = JobReadinessInhibitor(JobReadinessInhibitor.UNDESIRED)
+        obj = JobReadinessInhibitor(InhibitionCause.UNDESIRED)
         self.assertEqual(
             repr(obj), (
                 "<JobReadinessInhibitor cause:UNDESIRED"
@@ -66,7 +72,7 @@ class JobReadinessInhibitorTests(TestCase):
     def test_pending_dep(self):
         job = make_job("A")
         obj = JobReadinessInhibitor(
-            JobReadinessInhibitor.PENDING_DEP, related_job=job)
+            InhibitionCause.PENDING_DEP, related_job=job)
         self.assertEqual(
             repr(obj), (
                 "<JobReadinessInhibitor cause:PENDING_DEP"
@@ -77,7 +83,7 @@ class JobReadinessInhibitorTests(TestCase):
     def test_failed_dep(self):
         job = make_job("A")
         obj = JobReadinessInhibitor(
-            JobReadinessInhibitor.FAILED_DEP, related_job=job)
+            InhibitionCause.FAILED_DEP, related_job=job)
         self.assertEqual(
             repr(obj), (
                 "<JobReadinessInhibitor cause:FAILED_DEP"
@@ -89,7 +95,7 @@ class JobReadinessInhibitorTests(TestCase):
         job = make_job("A", requires="resource.attr == 'value'")
         expr = job.get_resource_program().expression_list[0]
         obj = JobReadinessInhibitor(
-            JobReadinessInhibitor.PENDING_RESOURCE, related_job=job,
+            InhibitionCause.PENDING_RESOURCE, related_job=job,
             related_expression=expr)
         self.assertEqual(
             repr(obj), (
@@ -107,7 +113,7 @@ class JobReadinessInhibitorTests(TestCase):
         job = make_job("A", requires="resource.attr == 'value'")
         expr = job.get_resource_program().expression_list[0]
         obj = JobReadinessInhibitor(
-            JobReadinessInhibitor.FAILED_RESOURCE, related_job=job,
+            InhibitionCause.FAILED_RESOURCE, related_job=job,
             related_expression=expr)
         self.assertEqual(
             repr(obj), (
@@ -122,7 +128,7 @@ class JobReadinessInhibitorTests(TestCase):
 
     def test_unknown_global(self):
         self.assertEqual(UndesiredJobReadinessInhibitor.cause,
-                         JobReadinessInhibitor.UNDESIRED)
+                         InhibitionCause.UNDESIRED)
 
 
 class JobStateTests(TestCase):
@@ -134,10 +140,14 @@ class JobStateTests(TestCase):
     def test_smoke(self):
         self.assertIsNotNone(self.job_state.result)
         self.assertIs(self.job_state.result.outcome, IJobResult.OUTCOME_NONE)
+        self.assertEqual(self.job_state.result_history, ())
         self.assertEqual(self.job_state.readiness_inhibitor_list, [
             UndesiredJobReadinessInhibitor])
         self.assertEqual(self.job_state.effective_category_id,
                          self.job.category_id)
+        self.assertEqual(self.job_state.effective_certification_status,
+                         self.job.certification_status)
+        self.assertIsNone(self.job_state.via_job)
 
     def test_getting_job(self):
         self.assertIs(self.job_state.job, self.job)
@@ -155,6 +165,16 @@ class JobStateTests(TestCase):
         self.job_state.result = result
         self.assertIs(self.job_state.result, result)
 
+    def test_result_history_keeps_track_of_result_changes(self):
+        # XXX: this example will fail if subsequent results are identical
+        self.assertEqual(self.job_state.result_history, ())
+        result1 = make_job_result(outcome='fail')
+        self.job_state.result = result1
+        self.assertEqual(self.job_state.result_history, (result1,))
+        result2 = make_job_result(outcome='pass')
+        self.job_state.result = result2
+        self.assertEqual(self.job_state.result_history, (result1, result2))
+
     def test_setting_result_fires_signal(self):
         """
         verify that assigning state.result fires the on_result_changed signal
@@ -195,7 +215,7 @@ class JobStateTests(TestCase):
         self.assertFalse(self.on_changed_fired)
 
     def test_setting_readiness_inhibitor_list(self):
-        inhibitor = JobReadinessInhibitor(JobReadinessInhibitor.UNDESIRED)
+        inhibitor = JobReadinessInhibitor(InhibitionCause.UNDESIRED)
         self.job_state.readiness_inhibitor_list = [inhibitor]
         self.assertEqual(self.job_state.readiness_inhibitor_list, [inhibitor])
 
@@ -219,3 +239,24 @@ class JobStateTests(TestCase):
     def test_setting_effective_category_id(self):
         self.job_state.effective_category_id = 'value'
         self.assertEqual(self.job_state.effective_category_id, 'value')
+
+    def test_setting_effective_cert_certification_status(self):
+        self.job_state.effective_certification_status = 'value'
+        self.assertEqual(self.job_state.effective_certification_status,
+                         'value')
+
+    def test_setting_via_job__TypeError(self):
+        with self.assertRaises(TypeError):
+            self.job_state.via_job = 'value'
+
+    def test_setting_via_job(self):
+        parent = make_job("parent")
+        self.job_state.via_job = parent
+        self.assertIs(self.job_state.via_job, parent)
+
+    def test_resetting_via_job(self):
+        parent = make_job("parent")
+        self.job_state.via_job = parent
+        self.assertIs(self.job_state.via_job, parent)
+        self.job_state.via_job = None
+        self.assertIs(self.job_state.via_job, None)
