@@ -25,7 +25,6 @@ Test definitions for plainbox.impl.session.manager module
 """
 
 from unittest import expectedFailure
-from unittest import TestCase
 
 from plainbox.abc import IJobDefinition
 from plainbox.impl.session import SessionManager
@@ -33,11 +32,11 @@ from plainbox.impl.session import SessionState
 from plainbox.impl.session import SessionStorage
 from plainbox.impl.session.state import SessionDeviceContext
 from plainbox.impl.session.suspend import SessionSuspendHelper
-from plainbox.impl.signal import SignalInterceptorMixIn
 from plainbox.vendor import mock
+from plainbox.vendor.morris import SignalTestCase
 
 
-class SessionManagerTests(TestCase, SignalInterceptorMixIn):
+class SessionManagerTests(SignalTestCase):
 
     def setUp(self):
         self.storage = mock.Mock(name="storage", spec=SessionStorage)
@@ -94,7 +93,8 @@ class SessionManagerTests(TestCase, SignalInterceptorMixIn):
             # Ensure that a fresh instance of the suspend helper was used to
             # call the suspend() method and that the session state parameter
             # was passed to it.
-            helper_cls().suspend.assert_called_with(self.context.state)
+            helper_cls().suspend.assert_called_with(
+                self.context.state, self.storage.location)
         # Ensure that save_checkpoint() was called on the storage object with
         # the return value of what the suspend helper produced.
         self.storage.save_checkpoint.assert_called_with(
@@ -113,7 +113,10 @@ class SessionManagerTests(TestCase, SignalInterceptorMixIn):
             resumed_state = mock.Mock(spec_set=SessionState)
             resumed_state.unit_list = unit_list
             helper_cls().resume.return_value = resumed_state
-            manager = SessionManager.load_session(unit_list, self.storage)
+            # NOTE: mock away _propagate_test_plans() so that we don't get
+            # unwanted side effects we're not testing here.
+            with mock.patch.object(SessionManager, '_propagate_test_plans'):
+                manager = SessionManager.load_session(unit_list, self.storage)
         # Ensure that the storage object was used to load the session snapshot
         self.storage.load_checkpoint.assert_called_with()
         # Ensure that the helper was instantiated with the unit list, flags and
@@ -136,6 +139,8 @@ class SessionManagerTests(TestCase, SignalInterceptorMixIn):
         verify that SessionManager.create() correctly sets up
         storage repository and creates session directories
         """
+        mocks['SessionStorage'].create.return_value = mock.MagicMock(
+            spec_set=SessionStorage)
         # Create the new manager
         manager = SessionManager.create()
         # Ensure that a default repository was created
@@ -164,6 +169,8 @@ class SessionManagerTests(TestCase, SignalInterceptorMixIn):
         verify that SessionManager.create_with_unit_list() correctly sets up
         storage repository and creates session directories
         """
+        mocks['SessionStorage'].create.return_value = mock.MagicMock(
+            spec_set=SessionStorage)
         # Mock unit list
         unit_list = mock.Mock(name='unit_list')
         # Create the new manager
@@ -191,12 +198,15 @@ class SessionManagerTests(TestCase, SignalInterceptorMixIn):
         SessionStorageRepository=mock.DEFAULT,
         SessionState=mock.DEFAULT,
         SessionStorage=mock.DEFAULT,
+        SessionDeviceContext=mock.DEFAULT,
         WellKnownDirsHelper=mock.DEFAULT)
     def test_create_with_state(self, **mocks):
         """
         verify that SessionManager.create_with_state() correctly sets up
         storage repository and creates session directories
         """
+        mocks['SessionStorage'].create.return_value = mock.MagicMock(
+            spec_set=SessionStorage)
         # Mock an empty list of units in teh session state object
         self.state.unit_list = []
         # Create the new manager
@@ -212,8 +222,13 @@ class SessionManagerTests(TestCase, SignalInterceptorMixIn):
         mocks['WellKnownDirsHelper'].assert_called_with(storage)
         helper = mocks['WellKnownDirsHelper']()
         helper.populate.assert_called_with()
+        # Ensure that the device context was created with the right state
+        # object
+        mocks['SessionDeviceContext'].assert_called_with(self.state)
         # Ensure that the resulting manager has correct data inside
-        self.assertEqual(manager.state, self.state)
+        self.assertEqual(
+            manager.device_context_list, [mocks['SessionDeviceContext']()])
+        # self.assertEqual(manager.state, self.state)
         self.assertEqual(manager.storage, storage)
 
     def test_add_device_context(self):
