@@ -42,6 +42,13 @@ from plainbox.impl.session.resume import CorruptedSessionError
 from plainbox.impl.session.resume import IncompatibleJobError
 from plainbox.impl.session.resume import IncompatibleSessionError
 from plainbox.impl.session.resume import ResumeDiscardQualifier
+from plainbox.impl.session.resume import SessionPeekHelper
+from plainbox.impl.session.resume import SessionPeekHelper1
+from plainbox.impl.session.resume import SessionPeekHelper2
+from plainbox.impl.session.resume import SessionPeekHelper3
+from plainbox.impl.session.resume import SessionPeekHelper4
+from plainbox.impl.session.resume import SessionPeekHelper5
+from plainbox.impl.session.resume import SessionPeekHelper6
 from plainbox.impl.session.resume import SessionResumeError
 from plainbox.impl.session.resume import SessionResumeHelper
 from plainbox.impl.session.resume import SessionResumeHelper1
@@ -49,6 +56,7 @@ from plainbox.impl.session.resume import SessionResumeHelper2
 from plainbox.impl.session.resume import SessionResumeHelper3
 from plainbox.impl.session.resume import SessionResumeHelper4
 from plainbox.impl.session.resume import SessionResumeHelper5
+from plainbox.impl.session.resume import SessionResumeHelper6
 from plainbox.impl.session.state import SessionState
 from plainbox.impl.testing_utils import make_job
 from plainbox.testing_utils.testcases import TestCaseWithParameters
@@ -117,60 +125,239 @@ class SessionResumeExceptionTests(TestCase):
 
 class SessionResumeHelperTests(TestCase):
 
-    @mock.patch('plainbox.impl.session.resume.SessionResumeHelper1')
-    def test_resume_dispatch_v1(self, mocked_helper1):
-        data = gzip.compress(
-            b'{"session":{"desired_job_list":[],"jobs":{},"metadata":'
-            b'{"app_blob":null,"flags":[],"running_job_name":null,"title":null'
-            b'},"results":{}},"version":1}')
-        SessionResumeHelper([], None, None).resume(data)
-        mocked_helper1.resume_json.assertCalledOnce()
+    def test_resume_dispatch_v1(self):
+        helper1 = SessionResumeHelper1
+        with mock.patch.object(helper1, 'resume_json'):
+            data = gzip.compress(
+                b'{"session":{"desired_job_list":[],"jobs":{},"metadata":'
+                b'{"app_blob":null,"flags":[],"running_job_name":null,'
+                b'"title":null},"results":{}},"version":1}')
+            SessionResumeHelper([], None, None).resume(data)
+            helper1.resume_json.assert_called_once_with(
+                {'session': {'jobs': {},
+                             'metadata': {'title': None,
+                                          'running_job_name': None,
+                                          'app_blob': None,
+                                          'flags': []},
+                             'desired_job_list': [],
+                             'results': {}},
+                 'version': 1}, None)
 
-    @mock.patch('plainbox.impl.session.resume.SessionResumeHelper2')
-    def test_resume_dispatch_v2(self, mocked_helper2):
-        data = gzip.compress(
-            b'{"session":{"desired_job_list":[],"jobs":{},"metadata":'
-            b'{"app_blob":null,"flags":[],"running_job_name":null,"title":null'
-            b'},"results":{}},"version":2}')
-        SessionResumeHelper([], None, None).resume(data)
-        mocked_helper2.resume_json.assertCalledOnce()
+    def test_resume_dispatch_v2(self):
+        helper2 = SessionResumeHelper2
+        with mock.patch.object(helper2, 'resume_json'):
+            data = gzip.compress(
+                b'{"session":{"desired_job_list":[],"jobs":{},"metadata":'
+                b'{"app_blob":null,"flags":[],"running_job_name":null,'
+                b'"title":null},"results":{}},"version":2}')
+            SessionResumeHelper([], None, None).resume(data)
+            helper2.resume_json.assert_called_once_with(
+                {'session': {'jobs': {},
+                             'metadata': {'title': None,
+                                          'running_job_name': None,
+                                          'app_blob': None,
+                                          'flags': []},
+                             'desired_job_list': [],
+                             'results': {}},
+                 'version': 2}, None)
 
-    @mock.patch('plainbox.impl.session.resume.SessionResumeHelper3')
-    def test_resume_dispatch_v3(self, mocked_helper3):
-        data = gzip.compress(
-            b'{"session":{"desired_job_list":[],"jobs":{},"metadata":'
-            b'{"app_blob":null,"app_id":null,"flags":[],'
-            b'"running_job_name":null,"title":null'
-            b'},"results":{}},"version":3}')
-        SessionResumeHelper([], None, None).resume(data)
-        mocked_helper3.resume_json.assertCalledOnce()
+    def test_resume_dispatch_v3(self):
+        helper3 = SessionResumeHelper3
+        with mock.patch.object(helper3, 'resume_json'):
+            data = gzip.compress(
+                b'{"session":{"desired_job_list":[],"jobs":{},"metadata":'
+                b'{"app_blob":null,"app_id":null,"flags":[],'
+                b'"running_job_name":null,"title":null'
+                b'},"results":{}},"version":3}')
+            SessionResumeHelper([], None, None).resume(data)
+            helper3.resume_json.assert_called_once_with(
+                {'session': {'jobs': {},
+                             'metadata': {'title': None,
+                                          'app_id': None,
+                                          'running_job_name': None,
+                                          'app_blob': None,
+                                          'flags': []},
+                             'desired_job_list': [],
+                             'results': {}},
+                 'version': 3}, None)
 
-    @mock.patch('plainbox.impl.session.resume.SessionResumeHelper4')
-    def test_resume_dispatch_v4(self, mocked_helper4):
-        data = gzip.compress(
-            b'{"session":{"desired_job_list":[],"jobs":{},"metadata":'
-            b'{"app_blob":null,"app_id":null,"flags":[],'
-            b'"running_job_name":null,"title":null'
-            b'},"results":{}},"version":4}')
-        SessionResumeHelper([], None, None).resume(data)
-        mocked_helper4.resume_json.assertCalledOnce()
+    def test_resume_dispatch_v4(self):
+        helper4 = SessionResumeHelper4
+        with mock.patch.object(helper4, 'resume_json'):
+            data = gzip.compress(
+                b'{"session":{"desired_job_list":[],"jobs":{},"metadata":'
+                b'{"app_blob":null,"app_id":null,"flags":[],'
+                b'"running_job_name":null,"title":null'
+                b'},"results":{}},"version":4}')
+            SessionResumeHelper([], None, None).resume(data)
+            helper4.resume_json.assert_called_once_with(
+                {'session': {'jobs': {},
+                             'metadata': {'title': None,
+                                          'app_id': None,
+                                          'running_job_name': None,
+                                          'app_blob': None,
+                                          'flags': []},
+                             'desired_job_list': [],
+                             'results': {}},
+                 'version': 4}, None)
 
-    @mock.patch('plainbox.impl.session.resume.SessionResumeHelper5')
-    def test_resume_dispatch_v5(self, mocked_helper5):
-        data = gzip.compress(
-            b'{"session":{"desired_job_list":[],"jobs":{},"metadata":'
-            b'{"app_blob":null,"app_id":null,"flags":[],'
-            b'"running_job_name":null,"title":null'
-            b'},"results":{}},"version":5}')
-        SessionResumeHelper([], None, None).resume(data)
-        mocked_helper5.resume_json.assertCalledOnce()
+    def test_resume_dispatch_v5(self):
+        helper5 = SessionResumeHelper5
+        with mock.patch.object(helper5, 'resume_json'):
+            data = gzip.compress(
+                b'{"session":{"desired_job_list":[],"jobs":{},"metadata":'
+                b'{"app_blob":null,"app_id":null,"flags":[],'
+                b'"running_job_name":null,"title":null'
+                b'},"results":{}},"version":5}')
+            SessionResumeHelper([], None, None).resume(data)
+            helper5.resume_json.assert_called_once_with(
+                {'session': {'jobs': {},
+                             'metadata': {'title': None,
+                                          'app_id': None,
+                                          'running_job_name': None,
+                                          'app_blob': None,
+                                          'flags': []},
+                             'desired_job_list': [],
+                             'results': {}},
+                 'version': 5}, None)
 
     def test_resume_dispatch_v6(self):
+        helper6 = SessionResumeHelper6
+        with mock.patch.object(helper6, 'resume_json'):
+            data = gzip.compress(
+                b'{"session":{"desired_job_list":[],"jobs":{},"metadata":'
+                b'{"app_blob":null,"app_id":null,"flags":[],'
+                b'"running_job_name":null,"title":null'
+                b'},"results":{}},"version":6}')
+            SessionResumeHelper([], None, None).resume(data)
+            helper6.resume_json.assert_called_once_with(
+                {'session': {'jobs': {},
+                             'metadata': {'title': None,
+                                          'app_id': None,
+                                          'running_job_name': None,
+                                          'app_blob': None,
+                                          'flags': []},
+                             'desired_job_list': [],
+                             'results': {}},
+                 'version': 6}, None)
+
+    def test_resume_dispatch_v7(self):
         data = gzip.compress(
-            b'{"version":6}')
+            b'{"version":7}')
         with self.assertRaises(IncompatibleSessionError) as boom:
             SessionResumeHelper([], None, None).resume(data)
-        self.assertEqual(str(boom.exception), "Unsupported version 6")
+        self.assertEqual(str(boom.exception), "Unsupported version 7")
+
+
+class SessionPeekHelperTests(TestCase):
+
+    def test_peek_dispatch_v1(self):
+        helper1 = SessionPeekHelper1
+        with mock.patch.object(helper1, 'peek_json'):
+            data = gzip.compress(
+                b'{"session":{"desired_job_list":[],"jobs":{},"metadata":'
+                b'{"app_blob":null,"flags":[],"running_job_name":null,'
+                b'"title":null},"results":{}},"version":1}')
+            SessionPeekHelper().peek(data)
+            helper1.peek_json.assert_called_once_with(
+                {'session': {'jobs': {},
+                             'metadata': {'title': None,
+                                          'running_job_name': None,
+                                          'app_blob': None,
+                                          'flags': []},
+                             'desired_job_list': [],
+                             'results': {}},
+                 'version': 1})
+
+    def test_peek_dispatch_v2(self):
+        helper2 = SessionPeekHelper2
+        with mock.patch.object(helper2, 'peek_json'):
+            data = gzip.compress(
+                b'{"session":{"desired_job_list":[],"jobs":{},"metadata":'
+                b'{"app_blob":null,"flags":[],"running_job_name":null,'
+                b'"title":null},"results":{}},"version":2}')
+            SessionPeekHelper().peek(data)
+            helper2.peek_json.assert_called_once_with(
+                {'session': {'jobs': {},
+                             'metadata': {'title': None,
+                                          'running_job_name': None,
+                                          'app_blob': None,
+                                          'flags': []},
+                             'desired_job_list': [],
+                             'results': {}},
+                 'version': 2})
+
+    def test_peek_dispatch_v3(self):
+        helper3 = SessionPeekHelper3
+        with mock.patch.object(helper3, 'peek_json'):
+            data = gzip.compress(
+                b'{"session":{"desired_job_list":[],"jobs":{},"metadata":'
+                b'{"app_blob":null,"flags":[],"running_job_name":null,'
+                b'"title":null},"results":{}},"version":3}')
+            SessionPeekHelper().peek(data)
+            helper3.peek_json.assert_called_once_with(
+                {'session': {'jobs': {},
+                             'metadata': {'title': None,
+                                          'running_job_name': None,
+                                          'app_blob': None,
+                                          'flags': []},
+                             'desired_job_list': [],
+                             'results': {}},
+                 'version': 3})
+
+    def test_peek_dispatch_v4(self):
+        helper4 = SessionPeekHelper4
+        with mock.patch.object(helper4, 'peek_json'):
+            data = gzip.compress(
+                b'{"session":{"desired_job_list":[],"jobs":{},"metadata":'
+                b'{"app_blob":null,"flags":[],"running_job_name":null,'
+                b'"title":null},"results":{}},"version":4}')
+            SessionPeekHelper().peek(data)
+            helper4.peek_json.assert_called_once_with(
+                {'session': {'jobs': {},
+                             'metadata': {'title': None,
+                                          'running_job_name': None,
+                                          'app_blob': None,
+                                          'flags': []},
+                             'desired_job_list': [],
+                             'results': {}},
+                 'version': 4})
+
+    def test_peek_dispatch_v5(self):
+        helper5 = SessionPeekHelper5
+        with mock.patch.object(helper5, 'peek_json'):
+            data = gzip.compress(
+                b'{"session":{"desired_job_list":[],"jobs":{},"metadata":'
+                b'{"app_blob":null,"flags":[],"running_job_name":null,'
+                b'"title":null},"results":{}},"version":5}')
+            SessionPeekHelper().peek(data)
+            helper5.peek_json.assert_called_once_with(
+                {'session': {'jobs': {},
+                             'metadata': {'title': None,
+                                          'running_job_name': None,
+                                          'app_blob': None,
+                                          'flags': []},
+                             'desired_job_list': [],
+                             'results': {}},
+                 'version': 5})
+
+    def test_peek_dispatch_v6(self):
+        helper6 = SessionPeekHelper6
+        with mock.patch.object(helper6, 'peek_json'):
+            data = gzip.compress(
+                b'{"session":{"desired_job_list":[],"jobs":{},"metadata":'
+                b'{"app_blob":null,"flags":[],"running_job_name":null,'
+                b'"title":null},"results":{}},"version":6}')
+            SessionPeekHelper().peek(data)
+            helper6.peek_json.assert_called_once_with(
+                {'session': {'jobs': {},
+                             'metadata': {'title': None,
+                                          'running_job_name': None,
+                                          'app_blob': None,
+                                          'flags': []},
+                             'desired_job_list': [],
+                             'results': {}},
+                 'version': 6})
 
 
 class SessionResumeTests(TestCase):
@@ -266,6 +453,7 @@ class EndToEndTests(TestCaseWithParameters):
                 }]
             },
             'desired_job_list': ['__category__', 'generator'],
+            'mandatory_job_list': [],
             'metadata': {
                 'flags': [],
                 'running_job_name': None,
@@ -349,7 +537,7 @@ class SessionStateResumeTests(TestCaseWithParameters):
     parameter_names = ('resume_cls',)
     parameter_values = ((SessionResumeHelper1,), (SessionResumeHelper2,),
                         (SessionResumeHelper3,), (SessionResumeHelper4,),
-                        (SessionResumeHelper5,))
+                        (SessionResumeHelper5,), (SessionResumeHelper6,))
 
     def setUp(self):
         self.session_repr = {}
@@ -361,8 +549,8 @@ class SessionStateResumeTests(TestCaseWithParameters):
         """
         with mock.patch.object(self.helper, attribute='_build_SessionState'):
             self.helper._build_SessionState(self.session_repr)
-            self.helper._build_SessionState.assertCalledOnceWith(
-                self.session_repr, None)
+            self.helper._build_SessionState.assert_called_once_with(
+                self.session_repr)
 
     def test_calls_restore_SessionState_jobs_and_results(self):
         """
@@ -370,11 +558,14 @@ class SessionStateResumeTests(TestCaseWithParameters):
         _build_SessionState().
         """
         mpo = mock.patch.object
-        with mpo(self.helper, '_build_SessionState'), \
-                mpo(self.helper, '_restore_SessionState_jobs_and_results'):
+        with mpo(self.helper, '_restore_SessionState_jobs_and_results'), \
+                mpo(self.helper, '_restore_SessionState_metadata'), \
+                mpo(self.helper, '_restore_SessionState_job_list'), \
+                mpo(self.helper, '_restore_SessionState_mandatory_job_list'), \
+                mpo(self.helper, '_restore_SessionState_desired_job_list'):
             session = self.helper._build_SessionState(self.session_repr)
             self.helper._restore_SessionState_jobs_and_results. \
-                assertCalledOnceWith(session, self.session_repr)
+                assert_called_once_with(session, self.session_repr)
 
     def test_calls_restore_SessionState_metadata(self):
         """
@@ -382,11 +573,14 @@ class SessionStateResumeTests(TestCaseWithParameters):
         _build_SessionState().
         """
         mpo = mock.patch.object
-        with mpo(self.helper, '_build_SessionState'), \
-                mpo(self.helper, '_restore_SessionState_metadata'):
+        with mpo(self.helper, '_restore_SessionState_jobs_and_results'), \
+                mpo(self.helper, '_restore_SessionState_metadata'), \
+                mpo(self.helper, '_restore_SessionState_job_list'), \
+                mpo(self.helper, '_restore_SessionState_mandatory_job_list'), \
+                mpo(self.helper, '_restore_SessionState_desired_job_list'):
             session = self.helper._build_SessionState(self.session_repr)
             self.helper._restore_SessionState_metadata. \
-                assertCalledOnceWith(session, self.session_repr)
+                assert_called_once_with(session.metadata, self.session_repr)
 
     def test_calls_restore_SessionState_desired_job_list(self):
         """
@@ -394,11 +588,14 @@ class SessionStateResumeTests(TestCaseWithParameters):
         _build_SessionState().
         """
         mpo = mock.patch.object
-        with mpo(self.helper, '_build_SessionState'), \
+        with mpo(self.helper, '_restore_SessionState_jobs_and_results'), \
+                mpo(self.helper, '_restore_SessionState_metadata'), \
+                mpo(self.helper, '_restore_SessionState_job_list'), \
+                mpo(self.helper, '_restore_SessionState_mandatory_job_list'), \
                 mpo(self.helper, '_restore_SessionState_desired_job_list'):
             session = self.helper._build_SessionState(self.session_repr)
             self.helper._restore_SessionState_desired_job_list. \
-                assertCalledOnceWith(session, self.session_repr)
+                assert_called_once_with(session, self.session_repr)
 
     def test_calls_restore_SessionState_job_list(self):
         """
@@ -406,10 +603,13 @@ class SessionStateResumeTests(TestCaseWithParameters):
         _build_SessionState().
         """
         mpo = mock.patch.object
-        with mpo(self.helper, '_build_SessionState'), \
-                mpo(self.helper, '_restore_SessionState_job_list'):
+        with mpo(self.helper, '_restore_SessionState_jobs_and_results'), \
+                mpo(self.helper, '_restore_SessionState_metadata'), \
+                mpo(self.helper, '_restore_SessionState_job_list'), \
+                mpo(self.helper, '_restore_SessionState_mandatory_job_list'), \
+                mpo(self.helper, '_restore_SessionState_desired_job_list'):
             session = self.helper._build_SessionState(self.session_repr)
-            self.helper._restore_SessionState_job_list.assertCalledOnceWith(
+            self.helper._restore_SessionState_job_list.assert_called_once_with(
                 session, self.session_repr)
 
 
@@ -424,7 +624,7 @@ class IOLogRecordResumeTests(TestCaseWithParameters):
     parameter_names = ('resume_cls',)
     parameter_values = ((SessionResumeHelper1,), (SessionResumeHelper2,),
                         (SessionResumeHelper3,), (SessionResumeHelper4,),
-                        (SessionResumeHelper5,))
+                        (SessionResumeHelper5,), (SessionResumeHelper6,))
 
     def test_build_IOLogRecord_missing_delay(self):
         """
@@ -715,7 +915,7 @@ class MemoryJobResultResumeTests(JobResultResumeMixIn, TestCaseWithParameters):
     parameter_names = ('resume_cls',)
     parameter_values = ((SessionResumeHelper1,), (SessionResumeHelper2,),
                         (SessionResumeHelper3,), (SessionResumeHelper4,),
-                        (SessionResumeHelper5,))
+                        (SessionResumeHelper5,), (SessionResumeHelper6,))
     good_repr = {
         'outcome': "pass",
         'comments': None,
@@ -790,7 +990,7 @@ class DiskJobResultResumeTestsCommon(JobResultResumeMixIn,
     parameter_names = ('resume_cls',)
     parameter_values = ((SessionResumeHelper1,), (SessionResumeHelper2,),
                         (SessionResumeHelper3,), (SessionResumeHelper4,),
-                        (SessionResumeHelper5,))
+                        (SessionResumeHelper5,), (SessionResumeHelper6,))
     good_repr = {
         'outcome': "pass",
         'comments': None,
@@ -892,7 +1092,7 @@ class DiskJobResultResumeTests5(TestCaseWithParameters):
     """ Tests for behavior of DiskJobResult resume for format 5. """
 
     parameter_names = ('resume_cls',)
-    parameter_values = ((SessionResumeHelper5,),)
+    parameter_values = ((SessionResumeHelper6,),)
     good_repr = {
         'outcome': "pass",
         'comments': None,
@@ -936,7 +1136,7 @@ class DesiredJobListResumeTests(TestCaseWithParameters):
     parameter_names = ('resume_cls',)
     parameter_values = ((SessionResumeHelper1,), (SessionResumeHelper2,),
                         (SessionResumeHelper3,), (SessionResumeHelper4,),
-                        (SessionResumeHelper5,))
+                        (SessionResumeHelper5,), (SessionResumeHelper6,))
 
     def setUp(self):
         # All of the tests need a SessionState object and some jobs to work
@@ -1313,7 +1513,7 @@ class ProcessJobTests(TestCaseWithParameters):
     parameter_names = ('resume_cls',)
     parameter_values = ((SessionResumeHelper1,), (SessionResumeHelper2,),
                         (SessionResumeHelper3,), (SessionResumeHelper4,),
-                        (SessionResumeHelper5,))
+                        (SessionResumeHelper5,), (SessionResumeHelper6,))
 
     def setUp(self):
         self.job_id = 'job'
@@ -1477,7 +1677,7 @@ class JobPluginSpecificTests(TestCaseWithParameters):
     parameter_names = ('resume_cls',)
     parameter_values = ((SessionResumeHelper1,), (SessionResumeHelper2,),
                         (SessionResumeHelper3,), (SessionResumeHelper4,),
-                        (SessionResumeHelper5,))
+                        (SessionResumeHelper5,), (SessionResumeHelper6,))
 
     def test_process_job_restores_resources(self):
         """
@@ -1579,7 +1779,7 @@ class SessionJobsAndResultsResumeTests(TestCaseWithParameters):
     parameter_names = ('resume_cls',)
     parameter_values = ((SessionResumeHelper1,), (SessionResumeHelper2,),
                         (SessionResumeHelper3,), (SessionResumeHelper4,),
-                        (SessionResumeHelper5,))
+                        (SessionResumeHelper5,), (SessionResumeHelper6,))
 
     def test_empty_session(self):
         """
@@ -1799,7 +1999,7 @@ class SessionJobListResumeTests(TestCaseWithParameters):
     parameter_names = ('resume_cls',)
     parameter_values = ((SessionResumeHelper1,), (SessionResumeHelper2,),
                         (SessionResumeHelper3,), (SessionResumeHelper4,),
-                        (SessionResumeHelper5,))
+                        (SessionResumeHelper5,), (SessionResumeHelper6,))
 
     def test_simple_session(self):
         """
@@ -1844,6 +2044,7 @@ class RegressionTests(TestCase):
             'session': {
                 'jobs': {},  # nothing ran yet
                 'desired_job_list': [job_a.id],  # we want a to run
+                'mandatory_job_list': [],
                 'results': {},  # nothing ran yet
             },
         }
@@ -1872,6 +2073,7 @@ class RegressionTests(TestCase):
                     job_a.id: job_a.checksum
                 },
                 'desired_job_list': [job_a.id],  # we want to run a
+                'mandatory_job_list': [],
                 'results': {},  # nothing ran yet
             }
         }
