@@ -32,9 +32,11 @@ quickly construct symbols without syntax overhead.
 
 __all__ = ['Symbol', 'SymbolDef']
 
+import functools
 import inspect
 
 
+@functools.total_ordering
 class Symbol:
     """
     Symbol type.
@@ -95,18 +97,18 @@ class Symbol:
         elif isinstance(other, str):
             return self._name == other
         else:
-            return False
+            return NotImplemented
 
-    def __ne__(self, other):
+    def __lt__(self, other):
         """
         Compare two symbols or a string and a symbol for inequality
         """
         if isinstance(other, Symbol):
-            return self is not other
+            return self._name < other._name
         elif isinstance(other, str):
-            return self._name != other
+            return self._name < other
         else:
-            return False
+            return NotImplemented
 
     def __hash__(self):
         """
@@ -121,13 +123,14 @@ class SymbolDefNs:
 
     A special namespace used by :class:`SymbolDefMeta` to keep track of names
     that were being accessed. Each accessed name is converted to a
-    :class:`Symbol` and added to the nanespace.
+    :class:`Symbol` and added to the namespace.
     """
 
     PASSTHRU = frozenset(('__name__', '__qualname__', '__doc__', '__module__'))
 
-    def __init__(self):
+    def __init__(self, allow_outer=None):
         self.data = {}
+        self.allow_outer = allow_outer
 
     def __setitem__(self, name, value):
         if name in self.PASSTHRU:
@@ -142,6 +145,8 @@ class SymbolDefNs:
     def __getitem__(self, name):
         if name in self.PASSTHRU:
             return self.data[name]
+        elif self.allow_outer is not None and name in self.allow_outer:
+            raise KeyError(name)
         elif name in self.data:
             return self.data[name]
         elif name == 'Symbol':
@@ -161,14 +166,17 @@ class SymbolDefMeta(type):
     """
 
     @classmethod
-    def __prepare__(mcls, name, bases, **kwargs):
-        return SymbolDefNs()
+    def __prepare__(mcls, name, bases, allow_outer=None, **kwargs):
+        return SymbolDefNs(allow_outer)
 
-    def __new__(mcls, name, bases, ns):
+    def __new__(mcls, name, bases, ns, allow_outer=None):
         classdict = ns.data
         classdict['get_all_symbols'] = classmethod(mcls.get_all_symbols)
         return type.__new__(mcls, name, bases, classdict)
 
+    def __init__(mcls, name, bases, ns, allow_outer=None):
+        super().__init__(name, bases, ns)
+
     # This is inserted via a simple trick because it's very hard to do any
     # normal method definition inside SymbolDef blocks.
     @staticmethod
