@@ -83,22 +83,29 @@ class CheckBoxSessionStateControllerTests(TestCase):
         self.assertEqual(
             self.ctrl.get_dependency_set(job_c),
             {('resource', 'j3')})
-        # Job with both direct and resource dependencies
+        # Job with ordering dependencies
         job_d = JobDefinition({
+            'after': 'j1, j2'
+        })
+        self.assertEqual(
+            self.ctrl.get_dependency_set(job_d),
+            {('ordering', 'j1'), ('ordering', 'j2')})
+        # Job with both direct and resource dependencies
+        job_e = JobDefinition({
             'depends': 'j4',
             'requires': 'j5.attr == 1'
         })
         self.assertEqual(
-            self.ctrl.get_dependency_set(job_d),
+            self.ctrl.get_dependency_set(job_e),
             {('direct', 'j4'), ('resource', 'j5')})
         # Job with both direct and resource dependencies
         # on the same job (j6)
-        job_e = JobDefinition({
+        job_f = JobDefinition({
             'depends': 'j6',
             'requires': 'j6.attr == 1'
         })
         self.assertEqual(
-            self.ctrl.get_dependency_set(job_e),
+            self.ctrl.get_dependency_set(job_f),
             {('direct', 'j6'), ('resource', 'j6')})
 
     def test_get_inhibitor_list_PENDING_RESOURCE(self):
@@ -162,23 +169,36 @@ class CheckBoxSessionStateControllerTests(TestCase):
             self.ctrl.get_inhibitor_list(session_state, j1), [])
 
     def test_get_inhibitor_list_PENDING_DEP(self):
-        # verify that jobs that depend on another job that hasn't
-        # been invoked yet produce the PENDING_DEP inhibitor
+        # verify that jobs that depend on another job or wait (via after) for
+        # another  that hasn't been invoked yet produce the PENDING_DEP
+        # inhibitor
         j1 = JobDefinition({
             'id': 'j1',
-            'depends': 'j2'
+            'depends': 'j2',
+            'after': 'j3',
         })
         j2 = JobDefinition({
             'id': 'j2'
         })
+        j3 = JobDefinition({
+            'id': 'j3'
+        })
         session_state = mock.MagicMock(spec=SessionState)
+        session_state.job_state_map = {
+            'j1': mock.Mock(spec_set=JobState),
+            'j2': mock.Mock(spec_set=JobState),
+            'j3': mock.Mock(spec_set=JobState),
+        }
         jsm_j2 = session_state.job_state_map['j2']
         jsm_j2.job = j2
         jsm_j2.result.outcome = IJobResult.OUTCOME_NONE
-        self.assertEqual(
-            self.ctrl.get_inhibitor_list(session_state, j1),
-            [JobReadinessInhibitor(
-                InhibitionCause.PENDING_DEP, j2, None)])
+        jsm_j3 = session_state.job_state_map['j3']
+        jsm_j3.job = j3
+        jsm_j3.result.outcome = IJobResult.OUTCOME_NONE
+        self.assertEqual(self.ctrl.get_inhibitor_list(session_state, j1), [
+            JobReadinessInhibitor(InhibitionCause.PENDING_DEP, j2, None),
+            JobReadinessInhibitor(InhibitionCause.PENDING_DEP, j3, None),
+        ])
 
     def test_get_inhibitor_list_FAILED_DEP(self):
         # verify that jobs that depend on another job that ran but
@@ -186,15 +206,27 @@ class CheckBoxSessionStateControllerTests(TestCase):
         # inhibitor.
         j1 = JobDefinition({
             'id': 'j1',
-            'depends': 'j2'
+            'depends': 'j2',
+            'after': 'j3',
         })
         j2 = JobDefinition({
             'id': 'j2'
         })
+        j3 = JobDefinition({
+            'id': 'j3'
+        })
         session_state = mock.MagicMock(spec=SessionState)
+        session_state.job_state_map = {
+            'j1': mock.Mock(spec_set=JobState),
+            'j2': mock.Mock(spec_set=JobState),
+            'j3': mock.Mock(spec_set=JobState),
+        }
         jsm_j2 = session_state.job_state_map['j2']
         jsm_j2.job = j2
         jsm_j2.result.outcome = IJobResult.OUTCOME_FAIL
+        jsm_j3 = session_state.job_state_map['j3']
+        jsm_j3.job = j3
+        jsm_j3.result.outcome = IJobResult.OUTCOME_FAIL
         self.assertEqual(
             self.ctrl.get_inhibitor_list(session_state, j1),
             [JobReadinessInhibitor(
@@ -205,15 +237,27 @@ class CheckBoxSessionStateControllerTests(TestCase):
         # equal to OUTCOME_PASS don't have any inhibitors
         j1 = JobDefinition({
             'id': 'j1',
-            'depends': 'j2'
+            'depends': 'j2',
+            'after': 'j3'
         })
         j2 = JobDefinition({
             'id': 'j2'
         })
+        j3 = JobDefinition({
+            'id': 'j3'
+        })
         session_state = mock.MagicMock(spec=SessionState)
+        session_state.job_state_map = {
+            'j1': mock.Mock(spec_set=JobState),
+            'j2': mock.Mock(spec_set=JobState),
+            'j3': mock.Mock(spec_set=JobState),
+        }
         jsm_j2 = session_state.job_state_map['j2']
         jsm_j2.job = j2
         jsm_j2.result.outcome = IJobResult.OUTCOME_PASS
+        jsm_j3 = session_state.job_state_map['j3']
+        jsm_j3.job = j3
+        jsm_j3.result.outcome = IJobResult.OUTCOME_PASS
         self.assertEqual(
             self.ctrl.get_inhibitor_list(session_state, j1), [])
 
@@ -226,7 +270,7 @@ class CheckBoxSessionStateControllerTests(TestCase):
         self.assertIs(
             session_state.job_state_map[job.id].result, result)
         # Ensure that signals got fired
-        session_state.on_job_state_map_changed.assert_called_once()
+        session_state.on_job_state_map_changed.assert_called_once_with()
         session_state.on_job_result_changed.assert_called_once_with(
             job, result)
 
@@ -686,7 +730,7 @@ class UserJobExecutionControllerTests(CheckBoxExecutionControllerTestsMixIn,
         self.assertEqual(self.ctrl.get_checkbox_score(self.job), 4)
 
     @mock.patch.dict('os.environ', clear=True)
-    def test_get_execution_environment_resets_LANG(self):
+    def test_get_execution_environment_resets_locales(self):
         # Call the tested method
         env = self.ctrl.get_execution_environment(
             self.job, self.job_state, self.config, self.SESSION_DIR,
@@ -694,15 +738,18 @@ class UserJobExecutionControllerTests(CheckBoxExecutionControllerTestsMixIn,
         # Ensure that LANG is reset to C.UTF-8
         self.assertEqual(env['LANG'], 'C.UTF-8')
 
-    @mock.patch.dict('os.environ', clear=True, LANG='fake_LANG')
-    def test_get_execution_environment_preserves_LANG_if_requested(self):
+    @mock.patch.dict('os.environ', clear=True, LANG='fake_LANG',
+                     LANGUAGE='fake_LANGUAGE', LC_ALL='fake_LC_ALL')
+    def test_get_execution_environment_preserves_locales_if_requested(self):
         self.job.get_flag_set.return_value = {'preserve-locale'}
         # Call the tested method
         env = self.ctrl.get_execution_environment(
             self.job, self.job_state, self.config, self.SESSION_DIR,
             self.NEST_DIR)
-        # Ensure that LANG is what we mocked it to be
+        # Ensure that locale variables are what we mocked them to be
         self.assertEqual(env['LANG'], 'fake_LANG')
+        self.assertEqual(env['LANGUAGE'], 'fake_LANGUAGE')
+        self.assertEqual(env['LC_ALL'], 'fake_LC_ALL')
 
     @mock.patch.dict('os.environ', clear=True, PYTHONPATH='PYTHONPATH')
     def test_get_execution_environment_keeps_PYTHONPATH(self):
@@ -822,6 +869,8 @@ class RootViaPTL1ExecutionControllerTests(
             '-G', 'CHECKBOX_DATA=session-dir/CHECKBOX_DATA',
             '-G', 'CHECKBOX_SHARE=CHECKBOX_SHARE-generator',
             '-G', 'LANG=C.UTF-8',
+            '-G', 'LANGUAGE=',
+            '-G', 'LC_ALL=C.UTF-8',
             '-G', 'PATH={}'.format(PATH),
             '-G', 'PLAINBOX_PROVIDER_DATA=data_dir-generator',
             '-G', 'PLAINBOX_PROVIDER_UNITS=units_dir-generator',
@@ -830,6 +879,8 @@ class RootViaPTL1ExecutionControllerTests(
             '-T', 'CHECKBOX_DATA=session-dir/CHECKBOX_DATA',
             '-T', 'CHECKBOX_SHARE=CHECKBOX_SHARE',
             '-T', 'LANG=C.UTF-8',
+            '-T', 'LANGUAGE=',
+            '-T', 'LC_ALL=C.UTF-8',
             '-T', 'PATH={}'.format(PATH),
             '-T', 'PLAINBOX_PROVIDER_DATA=data_dir',
             '-T', 'PLAINBOX_PROVIDER_UNITS=units_dir',
@@ -855,6 +906,8 @@ class RootViaPTL1ExecutionControllerTests(
             '-T', 'CHECKBOX_DATA=session-dir/CHECKBOX_DATA',
             '-T', 'CHECKBOX_SHARE=CHECKBOX_SHARE',
             '-T', 'LANG=C.UTF-8',
+            '-T', 'LANGUAGE=',
+            '-T', 'LC_ALL=C.UTF-8',
             '-T', 'PATH={}'.format(PATH),
             '-T', 'PLAINBOX_PROVIDER_DATA=data_dir',
             '-T', 'PLAINBOX_PROVIDER_UNITS=units_dir',
@@ -989,6 +1042,8 @@ class RootViaPkexecExecutionControllerTests(
              'CHECKBOX_DATA=session-dir/CHECKBOX_DATA',
              'CHECKBOX_SHARE=CHECKBOX_SHARE',
              'LANG=C.UTF-8',
+             'LANGUAGE=',
+             'LC_ALL=C.UTF-8',
              'PATH={}'.format(
                  os.pathsep.join([self.NEST_DIR, 'vanilla-path'])),
              'PLAINBOX_PROVIDER_DATA=data_dir',
@@ -1031,6 +1086,8 @@ class RootViaSudoExecutionControllerTests(
              'CHECKBOX_DATA=session-dir/CHECKBOX_DATA',
              'CHECKBOX_SHARE=CHECKBOX_SHARE',
              'LANG=C.UTF-8',
+             'LANGUAGE=',
+             'LC_ALL=C.UTF-8',
              'PATH={}'.format(
                  os.pathsep.join([self.NEST_DIR, 'vanilla-path'])),
              'PLAINBOX_PROVIDER_DATA=data_dir',
@@ -1134,9 +1191,9 @@ class QmlJobExecutionControllerTests(CheckBoxExecutionControllerTestsMixIn,
             self.ctrl.get_execution_command(
                 self.job, self.job_state, self.config, self.SESSION_DIR,
                 self.NEST_DIR, self.SHELL_OUT_FD, self.SHELL_IN_FD),
-            ['qmlscene', '--job', self.job.qml_file, '--fd-out',
-             self.SHELL_OUT_FD, '--fd-in', self.SHELL_IN_FD,
-             self.ctrl.QML_SHELL_PATH])
+            ['qmlscene', '-I', self.ctrl.QML_MODULES_PATH, '--job',
+             self.job.qml_file, '--fd-out', self.SHELL_OUT_FD, '--fd-in',
+             self.SHELL_IN_FD, self.ctrl.QML_SHELL_PATH])
 
     @mock.patch('json.dumps')
     @mock.patch('os.path.isdir')
