@@ -24,8 +24,6 @@ Module with implementation of legacy validation API for all the current units.
 This module can be removed once that API is no longer needed.
 """
 import itertools
-import re
-import sre_constants
 
 from plainbox.i18n import gettext as _
 from plainbox.impl import deprecated
@@ -401,11 +399,6 @@ class JobDefinitionLegacyAPI(UnitWithIdLegacyAPI):
             # very unreliable. There is no current demand for being able to
             # customize it from a resource record.
             'plugin': 'const',
-            # The command field should be variable if it is defined. This may
-            # be too strict but there has to be a channel between the resource
-            # object and the command to make the template job valuable so for
-            # now we will require variability here.
-            'command': 'vary',
             # The description should never be constant as that would be
             # confusing to the test operator. If it is defined in the template
             # it should be customized by at least one resource reference.
@@ -494,4 +487,4 @@ class TestPlanUnitLegacyAPI(UnitWithIdLegacyAPI):
         :attr:`Unit.Meta.template_constraints` field.
         """
         return TestPlanUnitValidatorLegacyAPI().validate(
-            self, **validation_kwargs)
\ No newline at end of file
+            self, **validation_kwargs)
