@@ -31,6 +31,8 @@ in a compatible way.
 import logging
 
 from plainbox.i18n import gettext as _
+from plainbox.i18n import gettext_noop as N_
+from plainbox.impl.symbol import SymbolDef
 from plainbox.impl.unit._legacy import CategoryUnitLegacyAPI
 from plainbox.impl.unit.unit_with_id import UnitWithId
 from plainbox.impl.unit.validators import CorrectFieldValueValidator
@@ -82,14 +84,6 @@ class CategoryUnit(UnitWithId, CategoryUnitLegacyAPI):
     def __repr__(self):
         return "<CategoryUnit id:{!r} name:{!r}>".format(self.id, self.name)
 
-    def tr_unit(self):
-        """
-        Translated name of the unit.
-
-        The return value is always 'category' (translated)
-        """
-        return _("category")
-
     @property
     def name(self):
         """
@@ -103,17 +97,17 @@ class CategoryUnit(UnitWithId, CategoryUnitLegacyAPI):
         """
         return self.get_translated_record_value("name")
 
-    class Meta(UnitWithId.Meta, CategoryUnitLegacyAPI.Meta):
+    class Meta:
 
-        class fields(UnitWithId.Meta.fields):
+        name = N_('category')
+
+        class fields(SymbolDef):
             """
             Symbols for each field that a JobDefinition can have
             """
             name = 'name'
 
-        field_validators = {}
-        field_validators.update(UnitWithId.Meta.field_validators)
-        field_validators.update({
+        field_validators = {
             fields.name: [
                 TranslatableFieldValidator,
                 TemplateVariantFieldValidator,
@@ -131,7 +125,4 @@ class CategoryUnit(UnitWithId, CategoryUnitLegacyAPI):
                     message=_("please stay under 80 characters"),
                     onlyif=lambda unit: unit.name is not None),
             ]
-        })
-
-
-CategoryUnit.fields = CategoryUnit.Meta.fields
+        }
