@@ -253,7 +253,10 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
 
     @propertywithsymbols(symbols=_PluginValues)
     def plugin(self):
-        return self.get_record_value('plugin')
+        plugin = self.get_record_value('plugin')
+        if plugin is None and 'simple' in self.get_flag_set():
+            plugin = 'shell'
+        return plugin
 
     @property
     def summary(self):
@@ -299,6 +302,10 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
     def depends(self):
         return self.get_record_value('depends')
 
+    @property
+    def after(self):
+        return self.get_record_value('after')
+
     @property
     def command(self):
         return self.get_record_value('command')
@@ -387,7 +394,33 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
         second.
         """
         value = self.get_record_value('estimated_duration')
-        if value is not None:
+        # NOTE: Some tests do that, I'd rather not change them now
+        if isinstance(value, (int, float)):
+            return value
+        elif value is None:
+            return None
+        match = re.match('^(\d+h)?[ :]*(\d+m)?[ :]*(\d+s)?$', value)
+        if match:
+            g_hours = match.group(1)
+            if g_hours:
+                assert g_hours.endswith('h')
+                hours = int(g_hours[:-1])
+            else:
+                hours = 0
+            g_minutes = match.group(2)
+            if g_minutes:
+                assert g_minutes.endswith('m')
+                minutes = int(g_minutes[:-1])
+            else:
+                minutes = 0
+            g_seconds = match.group(3)
+            if g_seconds:
+                assert g_seconds.endswith('s')
+                seconds = int(g_seconds[:-1])
+            else:
+                seconds = 0
+            return seconds + minutes * 60 + hours * 3600
+        else:
             try:
                 return float(value)
             except ValueError:
@@ -422,8 +455,8 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
             for stage in ['purpose', 'steps', 'verification']:
                 stage_value = self.get_translated_record_value(stage)
                 if stage_value is not None:
-                    tr_description += (tr_stages[stage] + ':\n'
-                                       + stage_value + '\n')
+                    tr_description += (tr_stages[stage] + ':\n' +
+                                       stage_value + '\n')
             tr_description = tr_description.strip()
             if not tr_description:
                 # combining new description yielded empty string
@@ -552,6 +585,36 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
         V().visit(WordList.parse(self.depends))
         return deps
 
+    def get_after_dependencies(self):
+        """
+        Compute and return a set of after dependencies.
+
+        After dependencies express the desire that given job A runs after a
+        given job B. This is spelled out as::
+
+            id: A
+            after: B
+
+            id: B
+
+        To combat a simple mistake where the jobs are space-delimited any
+        mixture of white-space (including newlines) and commas are allowed.
+        """
+        deps = set()
+        if self.after is None:
+            return deps
+
+        class V(Visitor):
+
+            def visit_Text_node(visitor, node: Text):
+                deps.add(self.qualify_id(node.text))
+
+            def visit_Error_node(visitor, node: Error):
+                logger.warning(_("unable to parse depends: %s"), node.msg)
+
+        V().visit(WordList.parse(self.after))
+        return deps
+
     def get_resource_dependencies(self):
         """
         Compute and return a set of resource dependencies
@@ -620,6 +683,7 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
             environ = 'environ'
             estimated_duration = 'estimated_duration'
             depends = 'depends'
+            after = 'after'
             requires = 'requires'
             shell = 'shell'
             imports = 'imports'
@@ -721,9 +785,12 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
                     severity=Severity.advice,
                     message=_("all jobs should have a description field, or a "
                               "set of purpose, steps and verification fields"),
-                    onlyif=lambda unit: unit.plugin != 'manual'
-                    and (unit.purpose is None and unit.steps is None and
-                         unit.verification is None)),
+                    onlyif=lambda unit: (
+                        'simple' not in unit.get_flag_set() and
+                        unit.plugin != 'manual' and (
+                            unit.purpose is None and
+                            unit.steps is None and
+                            unit.verification is None))),
             ],
             fields.purpose: [
                 TranslatableFieldValidator,
@@ -733,8 +800,8 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
                              " fields. See http://plainbox.readthedocs.org"
                              "/en/latest/author/faq.html#faq-2"),
                     onlyif=lambda unit:
-                    unit.startup_user_interaction_required
-                    and unit.get_record_value('summary') is None),
+                    unit.startup_user_interaction_required and
+                    unit.get_record_value('summary') is None),
             ],
             fields.steps: [
                 TranslatableFieldValidator,
@@ -750,11 +817,11 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
                 TranslatableFieldValidator,
                 PresentFieldValidator(
                     severity=Severity.advice,
-                    message = ("please use purpose, steps, and verification"
-                               " fields. See http://plainbox.readthedocs.org"
-                               "/en/latest/author/faq.html#faq-2"),
-                    onlyif=lambda unit:
-                    unit.startup_user_interaction_required),
+                    message=("please use purpose, steps, and verification"
+                             " fields. See http://plainbox.readthedocs.org"
+                             "/en/latest/author/faq.html#faq-2"),
+                    onlyif=lambda unit: unit.plugin in (
+                        'manual', 'user-verify', 'user-interact-verify')),
             ],
             fields.user: [
                 UntranslatableFieldValidator,
@@ -779,7 +846,10 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
             fields.estimated_duration: [
                 UntranslatableFieldValidator,
                 TemplateInvariantFieldValidator,
-                PresentFieldValidator(severity=Severity.advice),
+                PresentFieldValidator(
+                    severity=Severity.advice,
+                    onlyif=lambda unit: 'simple' not in unit.get_flag_set()
+                ),
                 CorrectFieldValueValidator(
                     lambda duration: float(duration) > 0,
                     message="value must be a positive number",
@@ -800,6 +870,18 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
                 # TODO: should not refer to deprecated jobs,
                 #       onlyif job itself is not deprecated
             ],
+            fields.after: [
+                UntranslatableFieldValidator,
+                CorrectFieldValueValidator(
+                    lambda value, unit: (
+                        unit.get_after_dependencies() is not None)),
+                UnitReferenceValidator(
+                    lambda unit: unit.get_after_dependencies(),
+                    constraints=[
+                        ReferenceConstraint(
+                            lambda referrer, referee: referee.unit == 'job',
+                            message=_("the referenced unit is not a job"))])
+            ],
             fields.requires: [
                 UntranslatableFieldValidator,
                 CorrectFieldValueValidator(
@@ -867,6 +949,7 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
                 TemplateInvariantFieldValidator,
                 CorrectFieldValueValidator(
                     lambda value, unit: (
+                        'simple' in unit.get_flag_set() or
                         'preserve-locale' in unit.get_flag_set()),
                     Problem.expected_i18n, Severity.advice,
                     message=_(
@@ -874,6 +957,25 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
                         ' non-C locale then set the preserve-locale flag'
                     ),
                     onlyif=lambda unit: unit.command),
+                CorrectFieldValueValidator(
+                    lambda value, unit: (
+                        not ('explicit-fail' in unit.get_flag_set() and
+                             unit.plugin in {
+                                 'shell', 'user-interact', 'attachment',
+                                 'local', 'resource'})),
+                    Problem.useless, Severity.advice,
+                    message=_('explicit-fail makes no sense for job which '
+                              'outcome is automatically determined.')
+                ),
+                # The has-leftovers flag is useless without a command
+                CorrectFieldValueValidator(
+                    lambda value, unit: (
+                        'has-leftovers' not in unit.get_flag_set()),
+                    Problem.useless, Severity.advice,
+                    message=_(
+                        'has-leftovers makes no sense without a command'
+                    ),
+                    onlyif=lambda unit: unit.command is None),
             ],
             fields.qml_file: [
                 UntranslatableFieldValidator,
@@ -884,13 +986,13 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
                     lambda value: value.endswith('.qml'),
                     Problem.wrong, Severity.advice,
                     message=_('use the .qml extension for all QML files'),
-                    onlyif=lambda unit: (unit.plugin == 'qml'
-                                         and unit.qml_file)),
+                    onlyif=lambda unit: (unit.plugin == 'qml' and
+                                         unit.qml_file)),
                 CorrectFieldValueValidator(
                     lambda value, unit: os.path.isfile(unit.qml_file),
                     message=_('please point to an existing QML file'),
-                    onlyif=lambda unit: (unit.plugin == 'qml'
-                                         and unit.qml_file)),
+                    onlyif=lambda unit: (unit.plugin == 'qml' and
+                                         unit.qml_file)),
             ],
             fields.certification_status: [
                 UntranslatableFieldValidator,
