@@ -578,6 +578,41 @@ class JobDefinitionFieldValidationTests(UnitWithIdFieldValidationTests):
             issue_list, self.unit_cls.Meta.fields.depends,
             Problem.bad_reference, Severity.error, message)
 
+    def test_after__untranslatable(self):
+        issue_list = self.unit_cls({
+            '_after': 'after'
+        }, provider=self.provider).check()
+        self.assertIssueFound(
+            issue_list, self.unit_cls.Meta.fields.after,
+            Problem.unexpected_i18n, Severity.warning)
+
+    def test_after__refers_to_other_units(self):
+        unit = self.unit_cls({
+            'after': 'some-unit'
+        }, provider=self.provider)
+        message = "field 'after', unit 'ns::some-unit' is not available"
+        self.provider.unit_list = [unit]
+        context = UnitValidationContext([self.provider])
+        issue_list = unit.check(context=context)
+        self.assertIssueFound(
+            issue_list, self.unit_cls.Meta.fields.after,
+            Problem.bad_reference, Severity.error, message)
+
+    def test_after__refers_to_other_jobs(self):
+        other_unit = UnitWithId({
+            'id': 'some-unit'
+        }, provider=self.provider)
+        unit = self.unit_cls({
+            'after': 'some-unit'
+        }, provider=self.provider)
+        message = "field 'after', the referenced unit is not a job"
+        self.provider.unit_list = [unit, other_unit]
+        context = UnitValidationContext([self.provider])
+        issue_list = unit.check(context=context)
+        self.assertIssueFound(
+            issue_list, self.unit_cls.Meta.fields.after,
+            Problem.bad_reference, Severity.error, message)
+
     def test_requires__untranslatable(self):
         issue_list = self.unit_cls({
             '_requires': 'requires'
@@ -701,6 +736,17 @@ class JobDefinitionFieldValidationTests(UnitWithIdFieldValidationTests):
             issue_list, self.unit_cls.Meta.fields.flags,
             Problem.expected_i18n, Severity.advice, message)
 
+    def test_flags__usless_explicit_fail_on_shell_jobs(self):
+        message = ("field 'flags', explicit-fail makes no sense for job which "
+                   "outcome is automatically determined.")
+        issue_list = self.unit_cls({
+            'plugin': 'shell',
+            'flags': 'explicit-fail'
+        }, provider=self.provider).check()
+        self.assertIssueFound(
+            issue_list, self.unit_cls.Meta.fields.flags,
+            Problem.useless, Severity.advice, message)
+
 
 class JobDefinitionValidatorTests(TestCase):
 
@@ -1075,13 +1121,6 @@ class TestJobDefinition(TestCase):
         observed = job.get_resource_dependencies()
         self.assertEqual(expected, observed)
 
-    def test_resource_parsing_broken(self):
-        job = JobDefinition({
-            'id': 'id',
-            'plugin': 'plugin',
-            'requires': "foo.bar == bar"})
-        self.assertRaises(Exception, job.get_resource_dependencies)
-
     def test_checksum_smoke(self):
         job1 = JobDefinition({
             'id': 'id',
@@ -1105,12 +1144,30 @@ class TestJobDefinition(TestCase):
             "cd21b33e6a2f4d1291977b60d922bbd276775adce73fca8c69b4821c96d7314a")
 
     def test_estimated_duration(self):
-        job1 = JobDefinition({})
-        self.assertEqual(job1.estimated_duration, None)
-        job2 = JobDefinition({'estimated_duration': 'foo'})
-        self.assertEqual(job2.estimated_duration, None)
-        job3 = JobDefinition({'estimated_duration': '123.5'})
-        self.assertEqual(job3.estimated_duration, 123.5)
+        self.assertEqual(JobDefinition({}).estimated_duration, None)
+        self.assertEqual(JobDefinition(
+            {'estimated_duration': 'foo'}).estimated_duration, None)
+        self.assertEqual(JobDefinition(
+            {'estimated_duration': '123.5'}).estimated_duration,
+            123.5)
+        self.assertEqual(JobDefinition(
+            {'estimated_duration': '5s'}).estimated_duration, 5)
+        self.assertEqual(JobDefinition(
+            {'estimated_duration': '1m 5s'}).estimated_duration, 65)
+        self.assertEqual(JobDefinition(
+            {'estimated_duration': '1h 1m 5s'}).estimated_duration, 3665)
+        self.assertEqual(JobDefinition(
+            {'estimated_duration': '1h'}).estimated_duration, 3600)
+        self.assertEqual(JobDefinition(
+            {'estimated_duration': '2m'}).estimated_duration, 120)
+        self.assertEqual(JobDefinition(
+            {'estimated_duration': '1h 1s'}).estimated_duration, 3601)
+        self.assertEqual(JobDefinition(
+            {'estimated_duration': '1m:5s'}).estimated_duration, 65)
+        self.assertEqual(JobDefinition(
+            {'estimated_duration': '1h:1m:5s'}).estimated_duration, 3665)
+        self.assertEqual(JobDefinition(
+            {'estimated_duration': '1h:1s'}).estimated_duration, 3601)
 
     def test_summary(self):
         job1 = JobDefinition({})
@@ -1357,5 +1414,5 @@ class RegressionTests(TestCase):
                 "manifest.has_thunderbolt == 'True'\n"),
         }, provider=provider)
         prog = job.get_resource_program()
-        self.assertEqual(prog.expression_list[-1].resource_id,
-                         '2013.com.canonical.plainbox::manifest')
+        self.assertEqual(prog.expression_list[-1].resource_id_list,
+                         ['2013.com.canonical.plainbox::manifest'])
