@@ -113,20 +113,6 @@ class TemplateUnitValidator(TestCase):
             boom.exception.field, JobDefinition.fields.description)
         self.assertEqual(boom.exception.problem, Problem.constant)
 
-    def test_checks_if_command_is_constant(self):
-        with self.assertRaises(ValidationError) as boom:
-            TemplateUnit({
-                'template-resource': 'resource',
-                'id': 'variable-{attr}',
-                'plugin': 'constant',
-                'summary': 'variable-{attr}',
-                'description': 'variable-{attr}',
-                'command': 'constant',
-            }).validate()
-        self.assertEqual(
-            boom.exception.field, JobDefinition.fields.command)
-        self.assertEqual(boom.exception.problem, Problem.constant)
-
     def test_checks_if_user_is_variable(self):
         with self.assertRaises(ValidationError) as boom:
             TemplateUnit({
@@ -480,7 +466,8 @@ class TemplateUnitFieldValidationTests(UnitFieldValidationTests):
         }, provider=self.provider)
         message = ("field 'template-resource',"
                    " unit 'ns::some-unit' is not available")
-        self.provider.get_units.return_value = ([unit], ())
+        self.provider.unit_list = [unit]
+        self.provider.problem_list = []
         context = UnitValidationContext([self.provider])
         issue_list = unit.check(context=context)
         self.assertIssueFound(
@@ -496,7 +483,8 @@ class TemplateUnitFieldValidationTests(UnitFieldValidationTests):
         }, provider=self.provider)
         message = ("field 'template-resource',"
                    " the referenced unit is not a job")
-        self.provider.get_units.return_value = ([unit, other_unit], ())
+        self.provider.unit_list = [unit, other_unit]
+        self.provider.problem_list = []
         context = UnitValidationContext([self.provider])
         issue_list = unit.check(context=context)
         self.assertIssueFound(
