@@ -93,6 +93,28 @@ class TestTestPlan(TestCase):
         }, provider=self.provider)
         self.assertEqual(unit.include, "include")
 
+    def test_mandatory_include__default(self):
+        unit = TestPlanUnit({
+        }, provider=self.provider)
+        self.assertEqual(unit.mandatory_include, None)
+
+    def test_mandatory_include__normal(self):
+        unit = TestPlanUnit({
+            'mandatory_include': 'mandatory_include'
+        }, provider=self.provider)
+        self.assertEqual(unit.mandatory_include, "mandatory_include")
+
+    def test_bootstrap_include__default(self):
+        unit = TestPlanUnit({
+        }, provider=self.provider)
+        self.assertEqual(unit.bootstrap_include, None)
+
+    def test_bootstrap_include__normal(self):
+        unit = TestPlanUnit({
+            'bootstrap_include': 'bootstrap_include'
+        }, provider=self.provider)
+        self.assertEqual(unit.bootstrap_include, 'bootstrap_include')
+
     def test_exclude__default(self):
         unit = TestPlanUnit({
         }, provider=self.provider)
@@ -139,10 +161,41 @@ class TestTestPlan(TestCase):
         self.assertEqual(unit.estimated_duration, None)
 
     def test_estimated_duration__normal(self):
-        unit = TestPlanUnit({
-            'estimated_duration': '5'
-        }, provider=self.provider)
-        self.assertEqual(unit.estimated_duration, 5)
+        self.assertEqual(TestPlanUnit(
+            {}, provider=self.provider).estimated_duration, None)
+        self.assertEqual(TestPlanUnit(
+            {'estimated_duration': '5'}, provider=self.provider
+        ).estimated_duration, 5)
+        self.assertEqual(TestPlanUnit(
+            {'estimated_duration': '123.5'}, provider=self.provider
+        ).estimated_duration, 123.5)
+        self.assertEqual(TestPlanUnit(
+            {'estimated_duration': '5s'}, provider=self.provider
+        ).estimated_duration, 5)
+        self.assertEqual(TestPlanUnit(
+            {'estimated_duration': '1m 5s'}, provider=self.provider
+        ).estimated_duration, 65)
+        self.assertEqual(TestPlanUnit(
+            {'estimated_duration': '1h 1m 5s'}, provider=self.provider
+        ).estimated_duration, 3665)
+        self.assertEqual(TestPlanUnit(
+            {'estimated_duration': '1h'}, provider=self.provider
+        ).estimated_duration, 3600)
+        self.assertEqual(TestPlanUnit(
+            {'estimated_duration': '2m'}, provider=self.provider
+        ).estimated_duration, 120)
+        self.assertEqual(TestPlanUnit(
+            {'estimated_duration': '1h 1s'}, provider=self.provider
+        ).estimated_duration, 3601)
+        self.assertEqual(TestPlanUnit(
+            {'estimated_duration': '1m:5s'}, provider=self.provider
+        ).estimated_duration, 65)
+        self.assertEqual(TestPlanUnit(
+            {'estimated_duration': '1h:1m:5s'}, provider=self.provider
+        ).estimated_duration, 3665)
+        self.assertEqual(TestPlanUnit(
+            {'estimated_duration': '1h:1s'}, provider=self.provider
+        ).estimated_duration, 3601)
 
     def test_estimated_duration__broken(self):
         unit = TestPlanUnit({
@@ -315,3 +368,20 @@ class TestTestPlan(TestCase):
         unit = TestPlanUnit({}, provider=self.provider)
         with self.assertRaisesRegex(ValueError, "expected override value"):
             unit.parse_category_overrides('apply')
+
+    def test_get_bootstrap_job_ids__empty(self):
+        unit = TestPlanUnit({}, provider=None)
+        self.assertEqual(unit.get_bootstrap_job_ids(), set())
+
+    def test_get_bootstrap_job_ids__normal(self):
+        unit = TestPlanUnit({
+            'bootstrap_include': 'Foo\nBar'
+        }, provider=None)
+        self.assertEqual(unit.get_bootstrap_job_ids(), set(['Foo', 'Bar']))
+
+    def test_get_bootstrap_job_ids__qualified_ids(self):
+        unit = TestPlanUnit({
+            'bootstrap_include': 'Foo\nBar'
+        }, provider=self.provider)
+        self.assertEqual(unit.get_bootstrap_job_ids(),
+                         set(['ns::Foo', 'ns::Bar']))
