@@ -36,20 +36,25 @@ from plainbox.impl.unit.unit_with_id import UnitWithId
 from plainbox.impl.unit.validators import CorrectFieldValueValidator
 from plainbox.impl.unit.validators import FieldValidatorBase
 from plainbox.impl.unit.validators import PresentFieldValidator
+from plainbox.impl.unit.validators import ReferenceConstraint
 from plainbox.impl.unit.validators import TemplateInvariantFieldValidator
 from plainbox.impl.unit.validators import TemplateVariantFieldValidator
 from plainbox.impl.unit.validators import TranslatableFieldValidator
+from plainbox.impl.unit.validators import UnitReferenceValidator
 from plainbox.impl.unit.validators import UntranslatableFieldValidator
 from plainbox.impl.unit.validators import compute_value_map
 from plainbox.impl.validation import Problem
 from plainbox.impl.validation import Severity
+from plainbox.impl.xparsers import Error
 from plainbox.impl.xparsers import FieldOverride
 from plainbox.impl.xparsers import IncludeStmt
 from plainbox.impl.xparsers import IncludeStmtList
 from plainbox.impl.xparsers import OverrideFieldList
 from plainbox.impl.xparsers import ReFixed
 from plainbox.impl.xparsers import RePattern
+from plainbox.impl.xparsers import Text
 from plainbox.impl.xparsers import Visitor
+from plainbox.impl.xparsers import WordList
 
 
 logger = logging.getLogger("plainbox.unit.testplan")
@@ -111,6 +116,72 @@ class NonEmptyPatternIntersectionValidator(FieldValidatorBase):
                 raise NotImplementedError
 
 
+class NoBaseIncludeValidator(FieldValidatorBase):
+    """
+    We want to ensure it does not select jobs already selected by the 'include'
+    field patterns.
+    """
+
+    def check_in_context(self, parent, unit, field, context):
+        for issue in self._check_test_plan_in_context(
+                parent, unit, field, context):
+            yield issue
+
+    def _check_test_plan_in_context(self, parent, unit, field, context):
+        included_job_id = []
+        id_map = context.compute_shared(
+            "field_value_map[id]", compute_value_map, context, 'id')
+        warning = _("selector {!a} will select a job already matched by the "
+                    "'include' field patterns")
+        qual_gen = unit._gen_qualifiers(
+            'include', getattr(unit, 'include'), True)
+        # Build the list of all jobs already included with the normal include
+        # field
+        for qual in qual_gen:
+            assert isinstance(qual, FieldQualifier)
+            if qual.field != 'id':
+                continue
+            if isinstance(qual.matcher, PatternMatcher):
+                for an_id in id_map:
+                    if an_id is None:
+                        continue
+                    if qual.matcher.match(an_id):
+                        included_job_id.append(an_id)
+            elif isinstance(qual.matcher, OperatorMatcher):
+                assert qual.matcher.op is operator.eq
+                target_id = qual.matcher.value
+                if target_id in id_map:
+                    included_job_id.append(target_id)
+            else:
+                raise NotImplementedError
+        # Now check that mandatory field patterns do not select a job already
+        # included with normal include.
+        qual_gen = unit._gen_qualifiers(
+            str(field), getattr(unit, str(field)), True)
+        for qual in qual_gen:
+            assert isinstance(qual, FieldQualifier)
+            if qual.field != 'id':
+                continue
+            if isinstance(qual.matcher, PatternMatcher):
+                for an_id in included_job_id:
+                    if qual.matcher.match(an_id):
+                        yield parent.warning(
+                            unit, field, Problem.bad_reference,
+                            warning.format(qual.matcher.pattern_text),
+                            origin=qual.origin)
+                        break
+            elif isinstance(qual.matcher, OperatorMatcher):
+                assert qual.matcher.op is operator.eq
+                target_id = qual.matcher.value
+                if target_id in included_job_id:
+                    yield parent.warning(
+                        unit, field, Problem.bad_reference,
+                        warning.format(target_id),
+                        origin=qual.origin)
+            else:
+                raise NotImplementedError
+
+
 class TestPlanUnit(UnitWithId, TestPlanUnitLegacyAPI):
     """
     Test plan class
@@ -154,6 +225,14 @@ class TestPlanUnit(UnitWithId, TestPlanUnitLegacyAPI):
     def include(self):
         return self.get_record_value('include')
 
+    @property
+    def mandatory_include(self):
+        return self.get_record_value('mandatory_include')
+
+    @property
+    def bootstrap_include(self):
+        return self.get_record_value('bootstrap_include')
+
     @property
     def exclude(self):
         return self.get_record_value('exclude')
@@ -181,8 +260,30 @@ class TestPlanUnit(UnitWithId, TestPlanUnitLegacyAPI):
         """
         value = self.get_record_value('estimated_duration')
         if value is None:
-            return
-        return float(value)
+            return None
+        match = re.match('^(\d+h)?[ :]*(\d+m)?[ :]*(\d+s)?$', value)
+        if match:
+            g_hours = match.group(1)
+            if g_hours:
+                assert g_hours.endswith('h')
+                hours = int(g_hours[:-1])
+            else:
+                hours = 0
+            g_minutes = match.group(2)
+            if g_minutes:
+                assert g_minutes.endswith('m')
+                minutes = int(g_minutes[:-1])
+            else:
+                minutes = 0
+            g_seconds = match.group(3)
+            if g_seconds:
+                assert g_seconds.endswith('s')
+                seconds = int(g_seconds[:-1])
+            else:
+                seconds = 0
+            return seconds + minutes * 60 + hours * 3600
+        else:
+            return float(value)
 
     def tr_name(self):
         """
@@ -196,6 +297,24 @@ class TestPlanUnit(UnitWithId, TestPlanUnitLegacyAPI):
         """
         return self.get_translated_record_value('description')
 
+    def get_bootstrap_job_ids(self):
+        """Compute and return a set of job ids from bootstrap_include field."""
+        job_ids = set()
+        if self.bootstrap_include is None:
+            return job_ids
+
+        class V(Visitor):
+
+            def visit_Text_node(visitor, node: Text):
+                job_ids.add(self.qualify_id(node.text))
+
+            def visit_Error_node(visitor, node: Error):
+                logger.warning(_(
+                    "unable to parse bootstrap_include: %s"), node.msg)
+
+        V().visit(WordList.parse(self.bootstrap_include))
+        return job_ids
+
     def get_qualifier(self):
         """
         Convert this test plan to an equivalent qualifier for job selection
@@ -207,6 +326,33 @@ class TestPlanUnit(UnitWithId, TestPlanUnitLegacyAPI):
         qual_list = []
         qual_list.extend(self._gen_qualifiers('include', self.include, True))
         qual_list.extend(self._gen_qualifiers('exclude', self.exclude, False))
+        qual_list.extend([self.get_bootstrap_qualifier(excluding=True)])
+        return CompositeQualifier(qual_list)
+
+    def get_mandatory_qualifier(self):
+        """
+        Convert this test plan to an equivalent qualifier for job selection
+
+        :returns:
+            A CompositeQualifier corresponding to the contents of both
+            the include and exclude fields.
+        """
+        qual_list = []
+        qual_list.extend(self._gen_qualifiers('include', self.mandatory_include, True))
+        return CompositeQualifier(qual_list)
+
+    def get_bootstrap_qualifier(self, excluding=False):
+        """
+        Convert this test plan to an equivalent qualifier for job selection
+        """
+        qual_list = []
+        if self.bootstrap_include is None:
+            return CompositeQualifier(qual_list)
+        field_origin = self.origin.just_line().with_offset(
+            self.field_offset_map['bootstrap_include'])
+        qual_list = [FieldQualifier(
+            'id', OperatorMatcher(operator.eq, target_id), field_origin,
+            not excluding) for target_id in self.get_bootstrap_job_ids()]
         return CompositeQualifier(qual_list)
 
     def _gen_qualifiers(self, field_name, field_value, inclusive):
@@ -398,6 +544,8 @@ class TestPlanUnit(UnitWithId, TestPlanUnitLegacyAPI):
             name = 'name'
             description = 'description'
             include = 'include'
+            mandatory_include = 'mandatory_include'
+            bootstrap_include = 'bootstrap_include'
             exclude = 'exclude'
             estimated_duration = 'estimated_duration'
             icon = 'icon'
@@ -430,6 +578,26 @@ class TestPlanUnit(UnitWithId, TestPlanUnitLegacyAPI):
             ],
             fields.include: [
                 NonEmptyPatternIntersectionValidator,
+                PresentFieldValidator(),
+            ],
+            fields.mandatory_include: [
+                NonEmptyPatternIntersectionValidator,
+                NoBaseIncludeValidator,
+            ],
+            fields.bootstrap_include: [
+                UntranslatableFieldValidator,
+                NoBaseIncludeValidator,
+                UnitReferenceValidator(
+                    lambda unit: unit.get_bootstrap_job_ids(),
+                    constraints=[
+                        ReferenceConstraint(
+                            lambda referrer, referee: referee.unit == 'job',
+                            message=_("the referenced unit is not a job")),
+                        ReferenceConstraint(
+                            lambda referrer, referee: (
+                                referee.plugin in ['local', 'resource']),
+                            message=_("only local and resource jobs are "
+                                      "allowed in bootstrapping_include"))])
             ],
             fields.exclude: [
                 NonEmptyPatternIntersectionValidator,
@@ -441,8 +609,7 @@ class TestPlanUnit(UnitWithId, TestPlanUnitLegacyAPI):
                     severity=Severity.advice,
                     onlyif=lambda unit: unit.virtual is False),
                 CorrectFieldValueValidator(
-                    lambda duration, unit: float(
-                        unit.get_record_value('estimated_duration')) > 0,
+                    lambda duration, unit: unit.estimated_duration > 0,
                     message="value must be a positive number",
                     onlyif=lambda unit: (
                         unit.virtual is False
