@@ -27,6 +27,7 @@ import itertools
 import logging
 import os
 import shlex
+import sys
 
 from plainbox.i18n import gettext as _
 from plainbox.i18n import ngettext
@@ -271,9 +272,12 @@ class CorrectFieldValueValidator(FieldValidatorBase):
             perform its check.
         """
         super().__init__(message)
-        spec = inspect.getargspec(correct_fn)
+        if sys.version_info[:2] >= (3, 5):
+            has_two_args = len(inspect.signature(correct_fn).parameters) == 2
+        else:
+            has_two_args = len(inspect.getargspec(correct_fn).args) == 2
         self.correct_fn = correct_fn
-        self.correct_fn_needs_unit = len(spec.args) == 2
+        self.correct_fn_needs_unit = has_two_args
         self.kind = kind or self.default_kind
         self.severity = severity or self.default_severity
         self.onlyif = onlyif
